@@ -18,9 +18,8 @@
 #include "config.h"
 
 #include <stdio.h>
-#include <X11/Xproto.h>
-#include <X11/Xatom.h>
 
+#include "libs/fvwm_x11.h"
 #include "libs/fvwmlib.h"
 #include "libs/FShape.h"
 #include "libs/Parse.h"
