@@ -11,19 +11,15 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 #include "config.h"
 
 #include <stdio.h>
 
-#include <X11/Xlib.h>
-#include <X11/Xmd.h>
-
+#include "libs/fvwm_x11.h"
 #include "libs/fvwmlib.h"
-#include "libs/FScreen.h"
 #include "libs/Strings.h"
 #include "fvwm.h"
 #include "externs.h"
@@ -52,21 +48,21 @@ extern ewmh_atom ewmh_atom_wm_state[];
 int ewmh_CurrentDesktop(
 	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
+	struct monitor	*m = monitor_get_current();
 	if (ev->xclient.data.l[0] < 0 || ev->xclient.data.l[0] > 0x7fffffff)
 	{
-		fvwm_msg(
-			WARN, "ewmh_CurrentDesktop",
-			"The application window (id %#lx)\n"
-			"  \"%s\" tried to switch to an invalid desktop (%ld)\n"
-			"  using an EWMH client message.\n"
-			"    fvwm is ignoring this request.\n",
-			fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
-			ev->xclient.data.l[0]);
+		fvwm_debug(__func__,
+			   "The application window (id %#lx)\n"
+			   "  \"%s\" tried to switch to an invalid desktop (%ld)\n"
+			   "  using an EWMH client message.\n"
+			   "    fvwm is ignoring this request.\n",
+			   fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
+			   ev->xclient.data.l[0]);
 		fvwm_msg_report_app_and_workers();
 
 		return -1;
 	}
-	goto_desk(ev->xclient.data.l[0]);
+	goto_desk(ev->xclient.data.l[0], m);
 
 	return -1;
 }
@@ -77,21 +73,21 @@ int ewmh_DesktopGeometry(
 	char action[256];
 	long width = ev->xclient.data.l[0];
 	long height = ev->xclient.data.l[1];
+	struct monitor	*m = monitor_get_current();
 
-	width = width / Scr.MyDisplayWidth;
-	height = height / Scr.MyDisplayHeight;
+	width = width / m->virtual_scr.MyDisplayWidth;
+	height = height / m->virtual_scr.MyDisplayHeight;
 
 	if (width <= 0 || height <= 0)
 	{
-		fvwm_msg(
-			WARN, "ewmh_DesktopGeometry",
-			"The application window (id %#lx)\n"
-			"  \"%s\" tried to set an invalid desktop geometry"
-			" (%ldx%ld)\n"
-			"  using an EWMH client message.\n"
-			"    fvwm is ignoring this request.\n",
-			fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
-			ev->xclient.data.l[0], ev->xclient.data.l[1]);
+		fvwm_debug(__func__,
+			   "The application window (id %#lx)\n"
+			   "  \"%s\" tried to set an invalid desktop geometry"
+			   " (%ldx%ld)\n"
+			   "  using an EWMH client message.\n"
+			   "    fvwm is ignoring this request.\n",
+			   fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
+			   ev->xclient.data.l[0], ev->xclient.data.l[1]);
 		fvwm_msg_report_app_and_workers();
 
 		return -1;
@@ -105,26 +101,27 @@ int ewmh_DesktopGeometry(
 int ewmh_DesktopViewPort(
 	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
+	struct monitor	*m = (fw && fw->m) ? fw->m : monitor_get_current();
+
 	if (
 		ev->xclient.data.l[0] < 0 ||
 		ev->xclient.data.l[0] > 0x7fffffff ||
 		ev->xclient.data.l[1] < 0 ||
 		ev->xclient.data.l[1] > 0x7fffffff)
 	{
-		fvwm_msg(
-			WARN, "ewmh_DesktopViewPort",
-			"The application window (id %#lx)\n"
-			"  \"%s\" tried to switch to an invalid page"
-			" (%ldx%ld)\n"
-			"  using an EWMH client message.\n"
-			"    fvwm is ignoring this request.\n",
-			fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
-			ev->xclient.data.l[0], ev->xclient.data.l[1]);
+		fvwm_debug(__func__,
+			   "The application window (id %#lx)\n"
+			   "  \"%s\" tried to switch to an invalid page"
+			   " (%ldx%ld)\n"
+			   "  using an EWMH client message.\n"
+			   "    fvwm is ignoring this request.\n",
+			   fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
+			   ev->xclient.data.l[0], ev->xclient.data.l[1]);
 		fvwm_msg_report_app_and_workers();
 
 		return -1;
 	}
-	MoveViewport(ev->xclient.data.l[0], ev->xclient.data.l[1], 1);
+	MoveViewport(m, ev->xclient.data.l[0], ev->xclient.data.l[1], 1);
 	return -1;
 }
 
@@ -132,25 +129,30 @@ int ewmh_NumberOfDesktops(
 	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	int d = ev->xclient.data.l[0];
+	struct monitor	*m;
 
 	/* not a lot of sinification for fvwm */
-	if (d > 0 && (d <= ewmhc.MaxDesktops || ewmhc.MaxDesktops == 0))
-	{
-		ewmhc.NumberOfDesktops = d;
-		EWMH_SetNumberOfDesktops();
-	}
-	else
-	{
-		fvwm_msg(
-			WARN, "ewmh_NumberOfDesktops",
-			"The application window (id %#lx)\n"
-			"  \"%s\" tried to set an invalid number of desktops"
-			" (%ld)\n"
-			"  using an EWMH client message.\n"
-			"    fvwm is ignoring this request.\n",
-			fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
-			ev->xclient.data.l[0]);
-		fvwm_msg_report_app_and_workers();
+	TAILQ_FOREACH(m, &monitor_q, entry) {
+		EWMH_SetNumberOfDesktops(m);
+
+		if (d > 0 &&
+		    (d <= m->ewmhc.MaxDesktops || m->ewmhc.MaxDesktops == 0))
+		{
+			m->ewmhc.NumberOfDesktops = d;
+		}
+		else
+		{
+			fvwm_debug(__func__,
+				   "The application window (id %#lx)\n"
+				   "  \"%s\" tried to set an invalid number of desktops"
+				   " (%ld)\n"
+				   "  using an EWMH client message.\n"
+				   "    fvwm is ignoring this request.\n",
+				   fw ? FW_W(fw) : 0,
+				   fw ? fw->name.name : "(none)",
+				   ev->xclient.data.l[0]);
+			fvwm_msg_report_app_and_workers();
+		}
 	}
 
 	return -1;
@@ -291,7 +293,7 @@ int ewmh_WMDesktop(
 			execute_function_override_window(
 				NULL, NULL, "Stick on", 0, fw);
 		}
-		else if (d > 0)
+		else if (d >= 0)
 		{
 			if (IS_STICKY_ACROSS_PAGES(fw) ||
 			    IS_STICKY_ACROSS_DESKS(fw))
@@ -306,16 +308,15 @@ int ewmh_WMDesktop(
 		}
 		else
 		{
-			fvwm_msg(
-				WARN, "ewmh_WMDesktop",
-				"The application window (id %#lx)\n"
-				"  \"%s\" tried to move to an invalid desk"
-				" (%ld)\n"
-				"  using an EWMH client message.\n"
-				"    fvwm is ignoring this request.\n",
-				fw ? FW_W(fw) : 0,
-				fw ? fw->name.name : "(none)",
-				ev->xclient.data.l[0]);
+			fvwm_debug(__func__,
+				   "The application window (id %#lx)\n"
+				   "  \"%s\" tried to move to an invalid desk"
+				   " (%ld)\n"
+				   "  using an EWMH client message.\n"
+				   "    fvwm is ignoring this request.\n",
+				   fw ? FW_W(fw) : 0,
+				   fw ? fw->name.name : "(none)",
+				   ev->xclient.data.l[0]);
 			fvwm_msg_report_app_and_workers();
 		}
 
@@ -349,11 +350,10 @@ int ewmh_WMDesktop(
 			return 0;
 		}
 #if DEBUG_EWMH_INIT_STATE
-		fprintf(
-			stderr, "ewmh WM_DESKTOP hint for window 0x%lx  "
-			"(%i,%lu,%u)\n", FW_W(fw),
-			HAS_EWMH_INIT_WM_DESKTOP(fw),
-			fw->ewmh_hint_desktop, val[0]);
+		fvwm_debug(__func__, "ewmh WM_DESKTOP hint for window 0x%lx  "
+			   "(%i,%lu,%u)\n", FW_W(fw),
+			   HAS_EWMH_INIT_WM_DESKTOP(fw),
+			   fw->ewmh_hint_desktop, val[0]);
 #endif
 		if (val[0] == (CARD32)-2 || val[0] == (CARD32)-1)
 		{
@@ -514,9 +514,8 @@ int ewmh_WMState(
 #if DEBUG_EWMH_INIT_STATE
 		if (size != 0)
 		{
-			fprintf(
-				stderr, "Window 0x%lx has an init"
-				" _NET_WM_STATE hint\n",FW_W(fw));
+			fvwm_debug(__func__, "Window 0x%lx has an init"
+				   " _NET_WM_STATE hint\n",FW_W(fw));
 		}
 #endif
 		nitems = size / sizeof(CARD32);
@@ -581,7 +580,7 @@ int ewmh_WMStateFullScreen(
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
 		{
-			fprintf(stderr,"\tFullscreen\n");
+			fvwm_debug(__func__, "\tFullscreen\n");
 		}
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
@@ -673,7 +672,7 @@ int ewmh_WMStateHidden(
 
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
-			fprintf(stderr,"\tHidden\n");
+			fvwm_debug(__func__, "\tHidden\n");
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
 		{
@@ -733,14 +732,15 @@ int ewmh_WMStateMaxHoriz(
 
 	if (ev == NULL && style == NULL)
 	{
-#if 0
-		return (IS_MAXIMIZED(fw) && !IS_EWMH_FULLSCREEN(fw));
-#else
-		/* DV: the notion of vertical/horizontal maximization does not
-		 * make any sense in fvwm, so just claim we're never maximized
+		/* If the window is maximized, but without any style, still
+		 * set the maximized status flag.
 		 */
-		return 0;
-#endif
+		bool maximized = (IS_MAXIMIZED(fw) && !IS_EWMH_FULLSCREEN(fw));
+
+		if (maximized)
+			SET_HAS_EWMH_INIT_MAXHORIZ_STATE(fw, EWMH_STATE_UNDEFINED_HINT);
+
+		return (maximized);
 	}
 
 	if (ev == NULL && style != NULL)
@@ -749,9 +749,8 @@ int ewmh_WMStateMaxHoriz(
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
 		{
-			fprintf(
-				stderr, "\t Maxhoriz %i\n",
-				HAS_EWMH_INIT_MAXHORIZ_STATE(fw));
+			fvwm_debug(__func__, "\t Maxhoriz %i\n",
+				   HAS_EWMH_INIT_MAXHORIZ_STATE(fw));
 		}
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
@@ -809,14 +808,14 @@ int ewmh_WMStateMaxVert(
 
 	if (ev == NULL && style == NULL)
 	{
-#if 0
-		return (IS_MAXIMIZED(fw) && !IS_EWMH_FULLSCREEN(fw));
-#else
-		/* DV: the notion of vertical/horizontal maximization does not
-		 * make any sense in fvwm, so just claim we're never maximized
+		/* If the window is maximized, but without any style, still
+		 * set the maximized status flag.
 		 */
-		return 0;
-#endif
+		bool maximized = (IS_MAXIMIZED(fw) && !IS_EWMH_FULLSCREEN(fw));
+
+		if (maximized)
+			SET_HAS_EWMH_INIT_MAXVERT_STATE(fw, EWMH_STATE_UNDEFINED_HINT);
+		return (maximized);
 	}
 
 	if (ev == NULL && style != NULL)
@@ -825,9 +824,8 @@ int ewmh_WMStateMaxVert(
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
 		{
-			fprintf(
-				stderr, "\t Maxvert %i\n",
-				HAS_EWMH_INIT_MAXVERT_STATE(fw));
+			fvwm_debug(__func__, "\t Maxvert %i\n",
+				   HAS_EWMH_INIT_MAXVERT_STATE(fw));
 		}
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
@@ -899,9 +897,8 @@ int ewmh_WMStateModal(
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
 		{
-			fprintf(
-				stderr, "\t Modal %i\n",
-				HAS_EWMH_INIT_MODAL_STATE(fw));
+			fvwm_debug(__func__, "\t Modal %i\n",
+				   HAS_EWMH_INIT_MODAL_STATE(fw));
 		}
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
@@ -1008,9 +1005,8 @@ int ewmh_WMStateShaded(
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
 		{
-			fprintf(
-				stderr, "\t Shaded %i\n",
-				HAS_EWMH_INIT_SHADED_STATE(fw));
+			fvwm_debug(__func__, "\t Shaded %i\n",
+				   HAS_EWMH_INIT_SHADED_STATE(fw));
 		}
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
@@ -1085,10 +1081,9 @@ int ewmh_WMStateSkipPager(
 		unsigned long has_hint = any;
 #if DEBUG_EWMH_INIT_STATE
 		/*if (has_hint)*/
-		fprintf(
-			stderr, "\t Skip_Pager %lu, %i, %i\n", has_hint,
-			HAS_EWMH_INIT_SKIP_PAGER_STATE(fw),
-			DO_EWMH_IGNORE_STATE_HINTS(style));
+		fvwm_debug(__func__, "\t Skip_Pager %lu, %i, %i\n", has_hint,
+			   HAS_EWMH_INIT_SKIP_PAGER_STATE(fw),
+			   DO_EWMH_IGNORE_STATE_HINTS(style));
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
 		{
@@ -1156,10 +1151,10 @@ int ewmh_WMStateSkipTaskBar(
 		unsigned long has_hint = any;
 #if DEBUG_EWMH_INIT_STATE
 		/*if (has_hint)*/
-		fprintf(stderr,"\t Skip_Taskbar %lu, %i, %i\n",
-			has_hint,
-			HAS_EWMH_INIT_SKIP_TASKBAR_STATE(fw),
-			DO_EWMH_IGNORE_STATE_HINTS(style));
+		fvwm_debug(__func__, "\t Skip_Taskbar %lu, %i, %i\n",
+			   has_hint,
+			   HAS_EWMH_INIT_SKIP_TASKBAR_STATE(fw),
+			   DO_EWMH_IGNORE_STATE_HINTS(style));
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
 		{
@@ -1232,7 +1227,7 @@ int ewmh_WMStateStaysOnTop(
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
 		{
-			fprintf(stderr,"\tStaysOnTop\n");
+			fvwm_debug(__func__, "\tStaysOnTop\n");
 		}
 #endif
 		if (!DO_EWMH_USE_STACKING_HINTS(style))
@@ -1319,7 +1314,7 @@ int ewmh_WMStateStaysOnBottom(
 		unsigned long has_hint = any;
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
-			fprintf(stderr,"\tStaysOnBottom\n");
+			fvwm_debug(__func__, "\tStaysOnBottom\n");
 #endif
 		if (!DO_EWMH_USE_STACKING_HINTS(style))
 		{
@@ -1407,7 +1402,7 @@ int ewmh_WMStateSticky(
 #if DEBUG_EWMH_INIT_STATE
 		if (has_hint)
 		{
-			fprintf(stderr,"\t Sticky\n");
+			fvwm_debug(__func__, "\t Sticky\n");
 		}
 #endif
 		if (DO_EWMH_IGNORE_STATE_HINTS(style))
@@ -1480,14 +1475,13 @@ int ewmh_WMIconGeometry(
 
 	if (val != NULL && size < 4 * sizeof(CARD32))
 	{
-		fvwm_msg(
-			WARN, "ewmh_WMIconGeometry",
-			"The application window (id %#lx)\n"
-			"  \"%s\" tried to set to an icon geometry via EWMH\n"
-			"  but provided only %d of the 4 values required.\n"
-			"    fvwm is ignoring this request.\n",
-			fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
-			(int)(size / sizeof(CARD32)));
+		fvwm_debug(__func__,
+			   "The application window (id %#lx)\n"
+			   "  \"%s\" tried to set to an icon geometry via EWMH\n"
+			   "  but provided only %d of the 4 values required.\n"
+			   "    fvwm is ignoring this request.\n",
+			   fw ? FW_W(fw) : 0, fw ? fw->name.name : "(none)",
+			   (int)(size / sizeof(CARD32)));
 		fvwm_msg_report_app_and_workers();
 		free(val);
 		val = NULL;
@@ -1534,6 +1528,9 @@ int ewmh_WMStrut(
 {
 	int size = 0;
 	CARD32 *val;
+	struct monitor	*m;
+
+	m = (fw && fw->m) ? fw->m : monitor_get_current();
 
 	if (ev == NULL)
 	{
@@ -1561,7 +1558,7 @@ int ewmh_WMStrut(
 		fw->strut.right  = val[1];
 		fw->strut.top    = val[2];
 		fw->strut.bottom = val[3];
-		ewmh_ComputeAndSetWorkArea();
+		ewmh_ComputeAndSetWorkArea(m);
 	}
 	if (val[0] !=  fw->dyn_strut.left ||
 	    val[1] != fw->dyn_strut.right ||
@@ -1572,7 +1569,7 @@ int ewmh_WMStrut(
 		fw->dyn_strut.right  = val[1];
 		fw->dyn_strut.top    = val[2];
 		fw->dyn_strut.bottom = val[3];
-		ewmh_HandleDynamicWorkArea();
+		ewmh_HandleDynamicWorkArea(m);
 	}
 	free(val);
 
