@@ -10,21 +10,21 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 /* ---------------------------- included header files ---------------------- */
 
 #define FEVENT_PRIVILEGED_ACCESS
 #include "config.h"
-#undef FEVENT_PRIVILEGED_ACCESS
 #include <stdio.h>
 
-#include "libs/safemalloc.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "execcontext.h"
+#include "libs/FEvent.h"
+
+#undef PEVENT_PRIVILEGED_ACCESS
 
 /* ---------------------------- local definitions -------------------------- */
 
@@ -99,12 +99,12 @@ const exec_context_t *exc_create_null_context(void)
 	int i;
 #endif
 
-	exc = (exec_context_t *)safecalloc(1, sizeof(exec_context_t));
+	exc = fxcalloc(1, sizeof(exec_context_t));
 #ifdef DEBUG_EXECCONTEXT
-fprintf(stderr, "xxx+0 ");
-for(i=0;i<nx;i++)fprintf(stderr,"  ");
+fvwm_debug(__func__, "xxx+0 ");
+for(i=0;i<nx;i++) fvwm_debug(__func__, "  ");
 x[nx]=exc;nx++;
-fprintf(stderr, "0x%08x\n", (int)exc);
+fvwm_debug(__func__, "0x%08x\n", (int)exc);
 #endif
 	exc->type = EXCT_NULL;
 	fev_make_null_event(&exc->private_data.te, dpy);
@@ -138,12 +138,12 @@ const exec_context_t *exc_clone_context(
 int i;
 #endif
 
-	exc = (exec_context_t *)safemalloc(sizeof(exec_context_t));
+	exc = fxmalloc(sizeof(exec_context_t));
 #ifdef DEBUG_EXECCONTEXT
-fprintf(stderr, "xxx+= ");
-for(i=0;i<nx;i++)fprintf(stderr,"  ");
+fvwm_debug(__func__, "xxx+= ");
+for(i=0;i<nx;i++) fvwm_debug(__func__, "  ");
 x[nx]=exc;nx++;
-fprintf(stderr, "0x%08x\n", (int)exc);
+fvwm_debug(__func__, "0x%08x\n", (int)exc);
 #endif
 	memcpy(exc, excin, sizeof(*exc));
 	__exc_change_context(exc, ecc, mask);
@@ -158,9 +158,9 @@ void exc_destroy_context(
 int i;
 if (nx == 0||x[nx-1] != exc)abort();
 nx--;
-fprintf(stderr, "xxx-- ");
-for(i=0;i<nx;i++)fprintf(stderr,"  ");
-fprintf(stderr, "0x%08x\n", (int)exc);
+fvwm_debug(__func__, "xxx-- ");
+for(i=0;i<nx;i++) fvwm_debug(__func__, "  ");
+fvwm_debug(__func__, "0x%08x\n", (int)exc);
 #endif
 	free((exec_context_t *)exc);
 
