@@ -178,6 +178,10 @@ const func_t func_table[] =
 	CMD_ENT("desk", CMD_Desk, F_GOTO_DESK, 0, 0),
 	/* - (obsolete, use GotoDesk instead) */
 
+	CMD_ENT("desktopconfiguration", CMD_DesktopConfiguration,
+		F_DESKTOP_CONFIGURATION, 0, 0),
+	/* - Defines how desktops are managed. */
+
 	CMD_ENT("desktopname", CMD_DesktopName, F_DESKTOP_NAME, 0, 0),
 	/* - Define the desktop names used in WindowList, modules */
 
@@ -464,13 +468,6 @@ const func_t func_table[] =
 	CMD_ENT("readwritecolors", CMD_ReadWriteColors, F_NOP, 0, 0),
 	/* - Used for colorset speed hacks (will be removed?) */
 
-	CMD_ENT("recapture", CMD_Recapture, F_RECAPTURE, 0, 0),
-	/* - Reapply styles to all windows (will be obsolete) */
-
-	CMD_ENT("recapturewindow", CMD_RecaptureWindow, F_RECAPTURE_WINDOW,
-		FUNC_NEEDS_WINDOW, CRS_SELECT),
-	/* - Reapply styles to one window (will be obsolete) */
-
 	CMD_ENT("refresh", CMD_Refresh, F_REFRESH, 0, 0),
 	/* - Cause all windows to redraw themselves */
 
@@ -566,6 +563,9 @@ const func_t func_table[] =
 		FUNC_NEEDS_WINDOW, CRS_SELECT),
 	/* - Control user defined window states */
 
+	CMD_ENT("status", CMD_Status, F_STATUS, 0, 0),
+	/* - Sets if we send status. */
+
 	CMD_ENT("stick", CMD_Stick, F_STICK,
 		FUNC_NEEDS_WINDOW, CRS_SELECT),
 	/* - Change window stickyness */
@@ -578,14 +578,6 @@ const func_t func_table[] =
 		FUNC_NEEDS_WINDOW, CRS_SELECT),
 	/* - Change window stickyness on a page basis */
 
-#ifdef HAVE_STROKE
-	CMD_ENT("stroke", CMD_Stroke, F_STROKE, 0, 0),
-	/* - Bind a stroke to an fvwm action */
-
-	CMD_ENT("strokefunc", CMD_StrokeFunc, F_STROKE_FUNC, 0, 0),
-	/* - Record stroke and execute corresponding stroke action */
-
-#endif /* HAVE_STROKE */
 	CMD_ENT("style", CMD_Style, F_STYLE, 0, 0),
 	/* - Set attributes of windows that match a pattern */
 
@@ -645,24 +637,6 @@ const func_t func_table[] =
 		FUNC_NEEDS_WINDOW, CRS_SELECT),
 	/* - Set styles on the selected window */
 
-	CMD_ENT("xinerama", CMD_Xinerama, F_XINERAMA, 0, 0),
-	/* - Control Xinerama support */
-
-	CMD_ENT("xineramaprimaryscreen", CMD_XineramaPrimaryScreen,
-		F_XINERAMAPRIMARYSCREEN, 0, 0),
-	/* - Identify Xinerama primary screen */
-
-	CMD_ENT("xineramasls", CMD_XineramaSls, F_XINERAMASLS, 0, 0),
-	/* - Control single logical screen support */
-
-	CMD_ENT("xineramaslsscreens", CMD_XineramaSlsScreens,
-		F_XINERAMASLSSCREENS, 0, 0),
-	/* - Configure layout for single logical screen */
-
-	CMD_ENT("xineramaslssize", CMD_XineramaSlsSize, F_XINERAMASLSSIZE,
-		0, 0),
-	/* - Configure layout for single logical screen sizes */
-
 	CMD_ENT("xorpixmap", CMD_XorPixmap, F_XOR, 0, 0),
 	/* - Use a pixmap for move/resize rubber-band */
 
