@@ -33,13 +33,13 @@
 
 /* ---------------------------- included header files ---------------------- */
 
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/cursorfont.h>
-#include <X11/Intrinsic.h>
+#include "libs/fvwm_x11.h"
 #include "libs/PictureBase.h"
+#include "libs/Fft.h"
+#include "libs/FRender.h"
 #include "libs/Flocale.h"
 #include "libs/fvwmrect.h"
+#include "libs/FScreen.h"
 #include "window_flags.h"
 #include "condrc.h"
 
@@ -78,9 +78,6 @@
 
 /* ---------------------------- forward declarations ----------------------- */
 
-struct size_borders;
-struct exec_context_t;
-
 /* definition in screen.h */
 struct FvwmDecor;
 
@@ -156,7 +153,7 @@ typedef struct icon_boxes_struct
 	struct icon_boxes_struct *next;       /* next icon_boxes or zero */
 	unsigned int use_count;
 	int IconBox[4];                       /* x/y x/y for iconbox */
-	int IconScreen;                       /* Xinerama screen */
+	char	*IconScreen;                       /* Xinerama screen */
 	short IconGrid[2];                    /* x incr, y incr */
 	char IconSign[4];                     /* because of -0, need to save */
 	unsigned is_orphan : 1;
@@ -466,7 +463,7 @@ typedef struct WindowConditionMask
 	struct name_condition *name_condition;
 	int layer;
 	int desk;
-	int screen;
+	struct monitor *screen;
 	int placed_by_button_mask;
 	int placed_by_button_set_mask;
 } WindowConditionMask;
@@ -600,6 +597,7 @@ typedef struct style_flags
 	unsigned has_initial_map_command_string : 1;
 	unsigned has_title_format_string : 1;
 	unsigned has_icon_title_format_string : 1;
+	unsigned initial_placement_done : 1;
 } style_flags;
 
 typedef struct style_id_t
@@ -678,7 +676,7 @@ typedef struct window_style
 	int start_desk;
 	int start_page_x;
 	int start_page_y;
-	int start_screen;
+	char *start_screen;
 	int min_window_width;
 	int min_window_height;
 	int max_window_width;
@@ -988,6 +986,9 @@ typedef struct FvwmWindow
 		void *p;
 		int i;
 	} scratch;
+
+	struct monitor *m;
+	struct monitor *m_prev;
 } FvwmWindow;
 
 /* ---------------------------- exported variables (globals) --------------- */
