@@ -27,16 +27,17 @@
 #endif
 
 #include "libs/fvwmlib.h"
-#include "libs/FScreen.h"
 #include "libs/FShape.h"
 #include "libs/Parse.h"
 #include "libs/Picture.h"
 #include "libs/Graphics.h"
 #include "libs/PictureGraphics.h"
 #include "libs/FRenderInit.h"
+#include "libs/FEvent.h"
 #include "libs/Rectangles.c"
 #include "libs/charmap.h"
 #include "libs/wcontext.h"
+#include "libs/safemalloc.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -401,7 +402,7 @@ ICON_DBG((stderr,"ciw: iph%s used '%s'\n",
 	{
 		int newWidth = fw->icon_g.picture_w_g.width;
 		int newHeight = fw->icon_g.picture_w_g.height;
-		Boolean resize = False;
+		Bool resize = False;
 
 		if (newWidth < fw->min_icon_width)
 		{
@@ -876,9 +877,8 @@ void DrawIconTitleWindow(
 		{
 			x_title_w = fw->icon_g.picture_w_g.x -
 				(w_title_w - fw->icon_g.picture_w_g.width) / 2;
-			FScreenGetScrRect(
-				NULL, FSCREEN_CURRENT,
-				&sx, &sy, &sw, &sh);
+			FScreenGetScrRect(NULL, FSCREEN_CURRENT,
+					  &sx, &sy, &sw, &sh);
 			/* start keep label on screen. dje 8/7/97 */
 			if (x_title_w < sx) {
 				/* if new loc neg (off left edge) */
@@ -1252,7 +1252,7 @@ void DrawIconWindow(
 	int co_title_cs = -1;
 	int is_expanded = IS_ICON_ENTERED(fw);
 
-	if (IS_ICON_SUPPRESSED(fw) || (pev && fw->Desk != Scr.CurrentDesk))
+	if (IS_ICON_SUPPRESSED(fw) || (pev && fw->Desk != fw->m->virtual_scr.CurrentDesk))
 	{
 		return;
 	}
@@ -1574,7 +1574,7 @@ void ChangeIconPixmap(FvwmWindow *fw)
 		{
 			LowerWindow(fw, False);
 			AutoPlaceIcon(fw, NULL, True);
-			if (fw->Desk == Scr.CurrentDesk)
+			if (fw->Desk == fw->m->virtual_scr.CurrentDesk)
 			{
 				if (FW_W_ICON_TITLE(fw))
 				{
@@ -1654,44 +1654,44 @@ void AutoPlaceIcon(
    * visible on the current page. */
   if (IS_ICON_STICKY_ACROSS_DESKS(t) || IS_STICKY_ACROSS_DESKS(t))
   {
-    t->Desk = Scr.CurrentDesk;
+    t->Desk = t->m->virtual_scr.CurrentDesk;
   }
   if (IS_ICON_STICKY_ACROSS_PAGES(t) || IS_STICKY_ACROSS_PAGES(t))
   {
     base_x = 0;
     base_y = 0;
     /*Also, if its a stickyWindow, put it on the current page! */
-    new_x = t->g.frame.x % Scr.MyDisplayWidth;
-    new_y = t->g.frame.y % Scr.MyDisplayHeight;
+    new_x = t->g.frame.x % t->m->virtual_scr.MyDisplayWidth;
+    new_y = t->g.frame.y % t->m->virtual_scr.MyDisplayHeight;
     if (new_x + t->g.frame.width <= 0)
-      new_x += Scr.MyDisplayWidth;
+      new_x += t->m->virtual_scr.MyDisplayWidth;
     if (new_y + t->g.frame.height <= 0)
-      new_y += Scr.MyDisplayHeight;
+      new_y += t->m->virtual_scr.MyDisplayHeight;
     frame_setup_window(
 	    t, new_x, new_y, t->g.frame.width, t->g.frame.height, False);
   }
-  else if (IsRectangleOnThisPage(&(t->g.frame), t->Desk))
+  else if (IsRectangleOnThisPage(t->m, &(t->g.frame), t->Desk))
   {
     base_x = 0;
     base_y = 0;
   }
   else
   {
-    base_x = ((t->g.frame.x + Scr.Vx + (t->g.frame.width >> 1)) /
-      Scr.MyDisplayWidth) * Scr.MyDisplayWidth;
-    base_y= ((t->g.frame.y + Scr.Vy + (t->g.frame.height >> 1)) /
-      Scr.MyDisplayHeight) * Scr.MyDisplayHeight;
+    base_x = ((t->g.frame.x + t->m->virtual_scr.Vx + (t->g.frame.width >> 1)) /
+      t->m->virtual_scr.MyDisplayWidth) * t->m->virtual_scr.MyDisplayWidth;
+    base_y= ((t->g.frame.y + t->m->virtual_scr.Vy + (t->g.frame.height >> 1)) /
+      t->m->virtual_scr.MyDisplayHeight) * t->m->virtual_scr.MyDisplayHeight;
     /* limit icon position to desktop */
-    if (base_x > Scr.VxMax)
-      base_x = Scr.VxMax;
+    if (base_x > t->m->virtual_scr.VxMax)
+      base_x = t->m->virtual_scr.VxMax;
     if (base_x < 0)
       base_x = 0;
-    if (base_y > Scr.VyMax)
-      base_y = Scr.VyMax;
+    if (base_y > t->m->virtual_scr.VyMax)
+      base_y = t->m->virtual_scr.VyMax;
     if (base_y < 0)
       base_y = 0;
-    base_x -= Scr.Vx;
-    base_y -= Scr.Vy;
+    base_x -= t->m->virtual_scr.Vx;
+    base_y -= t->m->virtual_scr.Vy;
   }
   if (IS_ICON_MOVED(t) ||
       (win_opts != NULL && win_opts->flags.use_initial_icon_xy))
@@ -1710,15 +1710,15 @@ void AutoPlaceIcon(
     dy = g.y;
 
     /* just make sure the icon is on this page */
-    g.x = g.x % Scr.MyDisplayWidth + base_x;
-    g.y = g.y % Scr.MyDisplayHeight + base_y;
+    g.x = g.x % t->m->virtual_scr.MyDisplayWidth + base_x;
+    g.y = g.y % t->m->virtual_scr.MyDisplayHeight + base_y;
     if (g.x < 0)
     {
-      g.x += Scr.MyDisplayWidth;
+      g.x += t->m->virtual_scr.MyDisplayWidth;
     }
     if (g.y < 0)
     {
-      g.y += Scr.MyDisplayHeight;
+      g.y += t->m->virtual_scr.MyDisplayHeight;
     }
     dx = g.x - dx;
     dy = g.y - dy;
@@ -1778,7 +1778,7 @@ void AutoPlaceIcon(
     dimension dim[3];                   /* space for work, 1st, 2nd dimen */
     icon_boxes *icon_boxes_ptr;         /* current icon box */
     int i;                              /* index for inner/outer loop data */
-    fscreen_scr_arg fscr;
+    fscreen_scr_arg *fscr;
     rectangle ref;
     rectangle g;
 
@@ -1792,8 +1792,9 @@ void AutoPlaceIcon(
 #define HRZ_FILL icon_boxes_ptr->IconFlags & ICONFILLHRZ
 
     /* needed later */
-    fscr.xypos.x = t->g.frame.x + (t->g.frame.width / 2) - base_x;
-    fscr.xypos.y = t->g.frame.y + (t->g.frame.height / 2) - base_y;
+    fscr = fxcalloc(1, sizeof *fscr);
+    fscr->xypos.x = t->g.frame.x + (t->g.frame.width / 2) - base_x;
+    fscr->xypos.y = t->g.frame.y + (t->g.frame.height / 2) - base_y;
     get_icon_geometry(t, &g);
     /* unnecessary copy of width */
     width = g.width;
@@ -1813,12 +1814,8 @@ void AutoPlaceIcon(
 	break;
       }
       /* get the screen dimensions for the icon box */
-      if (icon_boxes_ptr->IconScreen == FSCREEN_CURRENT)
-	fscr.mouse_ev = NULL;
-      FScreenGetScrRect(
-	&fscr,
-	icon_boxes_ptr->IconScreen,
-	&ref.x, &ref.y, &ref.width, &ref.height);
+      FScreenGetScrRect(fscr, FSCREEN_CURRENT,
+		        &ref.x, &ref.y, &ref.width, &ref.height);
       dim[1].screen_offset = ref.y;
       dim[1].screen_dimension = ref.height;
       dim[2].screen_offset = ref.x;
@@ -1965,7 +1962,7 @@ void AutoPlaceIcon(
 	  }
 
 	  /* this may be a good location */
-	  if (FScreenIsRectangleOnScreen(&fscr, FSCREEN_XYPOS, &ref))
+	  if (FScreenIsRectangleOnScreen(fscr, FSCREEN_XYPOS, &ref))
 	  {
 	    loc_ok = True;
 	  }
@@ -2018,6 +2015,8 @@ void AutoPlaceIcon(
     set_icon_position(t, real_x, real_y);
     broadcast_icon_geometry(t, True);
     do_move_icon = True;
+    free(fscr);
+
   }
   if (do_move_icon && do_move_immediately)
   {
@@ -2050,7 +2049,7 @@ do_all_iconboxes(FvwmWindow *t, icon_boxes **icon_boxes_ptr)
 		 * screen, uses an 80x80 grid, and fills top-bottom,
 		 * left-right */
 		FScreenGetScrRect(NULL, FSCREEN_PRIMARY, &sx, &sy, &sw, &sh);
-		global_icon_box_ptr = calloc(1, sizeof(icon_boxes));
+		global_icon_box_ptr = fxcalloc(1, sizeof(icon_boxes));
 		global_icon_box_ptr->IconBox[0] = sx;
 		global_icon_box_ptr->IconBox[1] = sy;
 		global_icon_box_ptr->IconBox[2] = sx + sw;
@@ -2126,7 +2125,7 @@ static void GetIconFromFile(FvwmWindow *fw)
 		&fw->iconDepth, &fw->icon_nalloc_pixels, &fw->icon_alloc_pixels,
 		&fw->icon_no_limit, fpa))
 	{
-		fvwm_msg(ERR, "GetIconFromFile", "Failed to load %s", path);
+		fvwm_debug(__func__, "Failed to load %s", path);
 		free(path);
 		return;
 	}
@@ -2161,10 +2160,9 @@ static void GetIconWindow(FvwmWindow *fw)
 		    (unsigned int*)&w, (unsigned int*)&h,(unsigned int*)&bw,
 		    (unsigned int*)&JunkDepth) == 0)
 	{
-		fvwm_msg(
-			ERR,"GetIconWindow",
-			"Window '%s' has a bad icon window! Ignoring icon"
-			" window.", fw->name.name);
+		fvwm_debug(__func__,
+			   "Window '%s' has a bad icon window! Ignoring icon"
+			   " window.", fw->name.name);
 		/* disable the icon window hint */
 		fw->wmhints->icon_window = None;
 		fw->wmhints->flags &= ~IconWindowHint;
@@ -2216,10 +2214,9 @@ static void GetIconBitmap(FvwmWindow *fw)
 		    (unsigned int*)&width, (unsigned int*)&height,
 		    (unsigned int*)&JunkBW, (unsigned int*)&depth))
 	{
-		fvwm_msg(
-			ERR,"GetIconBitmap",
-			"Window '%s' has a bad icon pixmap! Ignoring icon.",
-			fw->name.name);
+		fvwm_debug(__func__,
+			   "Window '%s' has a bad icon pixmap! Ignoring icon.",
+			   fw->name.name);
 		/* disable icon pixmap hint */
 		fw->wmhints->icon_pixmap = None;
 		fw->wmhints->flags &= ~IconPixmapHint;
@@ -2230,12 +2227,11 @@ static void GetIconBitmap(FvwmWindow *fw)
 	if (depth != 1 && depth != Pdepth &&
 	    depth != DefaultDepth(dpy,Scr.screen))
 	{
-		fvwm_msg(
-			ERR, "GetIconBitmap",
-			"Window '%s' has a bad icon bitmap depth %d (should"
-			" be 1, %d or %d)! Ignoring icon bitmap.",
-			fw->name.name, depth, Pdepth,
-			DefaultDepth(dpy,Scr.screen));
+		fvwm_debug(__func__,
+			   "Window '%s' has a bad icon bitmap depth %d (should"
+			   " be 1, %d or %d)! Ignoring icon bitmap.",
+			   fw->name.name, depth, Pdepth,
+			   DefaultDepth(dpy,Scr.screen));
 		/* disable icon pixmap hint */
 		fw->wmhints->icon_pixmap = None;
 		fw->wmhints->flags &= ~IconPixmapHint;
@@ -2343,7 +2339,7 @@ void DeIconify(FvwmWindow *fw)
 			XFlush(dpy);
 			/* End AS */
 			XMapWindow(dpy, FW_W(t));
-			if (t->Desk == Scr.CurrentDesk)
+			if (t->Desk == t->m->virtual_scr.CurrentDesk)
 			{
 				rectangle r;
 
@@ -2353,8 +2349,8 @@ void DeIconify(FvwmWindow *fw)
 				 * code already takes care of keeping the frame
 				 * geometry up to date */
 				update_absolute_geometry(t);
-				if (IsRectangleOnThisPage(&r, t->Desk) &&
-				    !IsRectangleOnThisPage(
+				if (IsRectangleOnThisPage(t->m, &r, t->Desk) &&
+				    !IsRectangleOnThisPage(t->m,
 					    &(t->g.frame), t->Desk))
 				{
 					/* Make sure we keep it on screen when
@@ -2362,11 +2358,11 @@ void DeIconify(FvwmWindow *fw)
 					t->g.frame.x -=
 						truncate_to_multiple(
 							t->g.frame.x,
-							Scr.MyDisplayWidth);
+							t->m->virtual_scr.MyDisplayWidth);
 					t->g.frame.y -=
 						truncate_to_multiple(
 							t->g.frame.y,
-							Scr.MyDisplayHeight);
+							t->m->virtual_scr.MyDisplayHeight);
 					XMoveWindow(
 						dpy, FW_W_FRAME(t),
 						t->g.frame.x, t->g.frame.y);
@@ -2408,7 +2404,7 @@ void DeIconify(FvwmWindow *fw)
 					(long)icon_rect.height);
 			}
 			XMapWindow(dpy, FW_W_PARENT(t));
-			if (t->Desk == Scr.CurrentDesk)
+			if (t->Desk == t->m->virtual_scr.CurrentDesk)
 			{
 				XMapWindow(dpy, FW_W_FRAME(t));
 				SET_MAP_PENDING(t, 1);
@@ -2621,9 +2617,9 @@ void Iconify(FvwmWindow *fw, initial_window_options_t *win_opts)
 	}
 	if (IS_ICON_STICKY_ACROSS_DESKS(fw) || IS_STICKY_ACROSS_DESKS(fw))
 	{
-		fw->Desk = Scr.CurrentDesk;
+		fw->Desk = fw->m->virtual_scr.CurrentDesk;
 	}
-	if (fw->Desk == Scr.CurrentDesk)
+	if (fw->Desk == fw->m->virtual_scr.CurrentDesk)
 	{
 		if (FW_W_ICON_TITLE(fw) != None)
 		{
