@@ -10,16 +10,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
-
-/*
- * This module is all original code
- * by Rob Nation
- * Copyright 1993, Robert Nation
- *     You may use this code for any purpose, as long as the original
- *     copyright remains in the source code and all documentation
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 /* ---------------------------- included header files ---------------------- */
@@ -29,10 +20,11 @@
 #include <stdio.h>
 #include <stdarg.h>
 
+#include "libs/fvwm_x11.h"
 #include "libs/ftime.h"
-#ifdef FVWM_DEBUG_TIME
-#include <sys/times.h>
-#endif
+#include "libs/Parse.h"
+#include "libs/Target.h"
+#include "libs/FEvent.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "execcontext.h"
@@ -85,8 +77,9 @@ static void change_grab_cursor(int cursor)
 int GetTwoArguments(
 	char *action, int *val1, int *val2, int *val1_unit, int *val2_unit)
 {
-	*val1_unit = Scr.MyDisplayWidth;
-	*val2_unit = Scr.MyDisplayHeight;
+	struct monitor	*m = monitor_get_current();
+	*val1_unit = m->virtual_scr.MyDisplayWidth;
+	*val2_unit = m->virtual_scr.MyDisplayHeight;
 
 	return GetTwoPercentArguments(action, val1, val2, val1_unit, val2_unit);
 }
@@ -106,12 +99,12 @@ void print_grab_stats(char *text)
 {
 	int i;
 
-	fprintf(stderr,"grab_stats (%s):", text);
+	fvwm_debug(__func__, "grab_stats (%s):", text);
 	for (i = 0; i < GRAB_MAXVAL; i++)
 	{
-		fprintf(stderr," %d", grab_count[i]);
+		fvwm_debug(__func__, " %d", grab_count[i]);
 	}
-	fprintf(stderr," \n");
+	fvwm_debug(__func__, " \n");
 
 	return;
 }
@@ -127,9 +120,8 @@ Bool GrabEm(int cursor, int grab_context)
 
 	if (grab_context <= GRAB_STARTUP || grab_context >= GRAB_MAXVAL)
 	{
-		fvwm_msg(
-			ERR, "GrabEm", "Bug: Called with illegal context %d",
-			grab_context);
+		fvwm_debug(__func__, "Bug: Called with illegal context %d",
+			   grab_context);
 		return False;
 	}
 
@@ -256,9 +248,8 @@ Bool UngrabEm(int ungrab_context)
 {
 	if (ungrab_context <= GRAB_ALL || ungrab_context >= GRAB_MAXVAL)
 	{
-		fvwm_msg(
-			ERR, "UngrabEm", "Bug: Called with illegal context %d",
-			ungrab_context);
+		fvwm_debug(__func__, "Bug: Called with illegal context %d",
+			   ungrab_context);
 		return False;
 	}
 
@@ -327,112 +318,25 @@ Bool UngrabEm(int ungrab_context)
 	return True;
 }
 
-#ifndef fvwm_msg /* Some ports (i.e. VMS) define their own version */
-/*
-** fvwm_msg: used to send output from fvwm to files and or stderr/stdout
-**
-** type -> DBG == Debug, ERR == Error, INFO == Information, WARN == Warning,
-** OLD == Command or option deprecated
-** id -> name of function, or other identifier
-*/
-static char *fvwm_msg_strings[] =
-{
-	"<<DEBUG>> ", "", "", "<<WARNING>> ", "<<DEPRECATED>> ", "<<ERROR>> "
-};
-
-void fvwm_msg(fvwm_msg_t type, char *id, char *msg, ...)
-{
-	va_list args;
-	char fvwm_id[20];
-	char time_str[40] = "\0";
-#ifdef FVWM_DEBUG_TIME
-	clock_t time_val, time_taken;
-	static clock_t start_time = 0;
-	static clock_t prev_time = 0;
-	struct tms not_used_tms;
-	time_t mytime;
-	struct tm *t_ptr;
-#endif
-
-#ifdef FVWM_DEBUG_TIME
-	time(&mytime);
-	t_ptr = localtime(&mytime);
-	if (start_time == 0)
-	{
-		/* get clock ticks */
-		prev_time = start_time = (unsigned int)times(&not_used_tms);
-	}
-	time_val = (unsigned int)times(&not_used_tms); /* get clock ticks */
-	time_taken = time_val - prev_time;
-	prev_time = time_val;
-	sprintf(time_str, "%.2d:%.2d:%.2d%7ld ",
-		t_ptr->tm_hour, t_ptr->tm_min, t_ptr->tm_sec, time_taken);
-#endif
-
-	strcpy(fvwm_id, "FVWM");
-	if (Scr.NumberOfScreens > 1)
-	{
-		sprintf(&fvwm_id[strlen(fvwm_id)], ".%d", (int)Scr.screen);
-	}
-
-	fprintf(stderr, "%s[%s][%s]: %s",
-		time_str, fvwm_id, id, fvwm_msg_strings[(int)type]);
-
-	if (type == ECHO)
-	{
-		/* user echos must be printed as a literal string */
-		fprintf(stderr, "%s", msg);
-	}
-	else
-	{
-		va_start(args, msg);
-		vfprintf(stderr, msg, args);
-		va_end(args);
-	}
-	fprintf(stderr, "\n");
-
-	if (type == ERR)
-	{
-		/* I hate to use a fixed length but this will do for now */
-		char tmp[2 * MAX_TOKEN_LENGTH];
-		sprintf(tmp, "[%s][%s]: %s",
-			fvwm_id, id, fvwm_msg_strings[(int)type]);
-		va_start(args, msg);
-		vsprintf(tmp + strlen(tmp), msg, args);
-		va_end(args);
-		tmp[strlen(tmp) + 1] = '\0';
-		tmp[strlen(tmp)] = '\n';
-		if (strlen(tmp) >= MAX_MODULE_INPUT_TEXT_LEN)
-		{
-			sprintf(tmp + MAX_MODULE_INPUT_TEXT_LEN - 5, "...\n");
-		}
-		BroadcastName(M_ERROR, 0, 0, 0, tmp);
-	}
-
-} /* fvwm_msg */
-#endif
-
 void fvwm_msg_report_app(void)
 {
-	fprintf(
-		stderr,
-		"    If you are having a problem with the application, send a"
-		" bug report\n"
-		"    with this message included to the application owner.\n"
-		"    There is no need to notify fvwm-workers@fvwm.org.\n");
+	fvwm_debug(__func__,
+		   "    If you are having a problem with the application, send a"
+		   " bug report\n"
+		   "    with this message included to the application owner.\n"
+		   "    There is no need to notify fvwm-workers@fvwm.org.\n");
 
 	return;
 }
 
 void fvwm_msg_report_app_and_workers(void)
 {
-	fprintf(
-		stderr,
-		"    If you are having a problem with the application, send"
-		" a bug report with\n"
-		"    this message included to the application owner and"
-		" notify\n"
-		"    fvwm-workers@fvwm.org.\n");
+	fvwm_debug(__func__,
+		   "    If you are having a problem with the application, send"
+		   " a bug report with\n"
+		   "    this message included to the application owner and"
+		   " notify\n"
+		   "    fvwm-workers@fvwm.org.\n");
 
 	return;
 }
@@ -513,14 +417,29 @@ int truncate_to_multiple (int x, int m)
 	return (x < 0) ? (m * (((x + 1) / m) - 1)) : (m * (x / m));
 }
 
-Bool IsRectangleOnThisPage(const rectangle *rec, int desk)
+Bool IsRectangleOnThisPage(struct monitor *m, const rectangle *rec, int desk)
 {
-	return (desk == Scr.CurrentDesk &&
-		rec->x + (signed int)rec->width > 0 &&
-		(rec->x < 0 || rec->x < Scr.MyDisplayWidth) &&
+#if 0
+	fprintf(stderr, "%s: I think I want monitor '%s'\n", __func__,
+		m->name);
+	fprintf(stderr, "%s: fw desk is: %d, mon desk is: %d\n", __func__,
+		desk, m->virtual_scr.CurrentDesk);
+	fprintf(stderr, "%s: fw rec: {x: %d, y: %d, w: %d, h: %d}\n", __func__,
+		rec->x, rec->y, rec->width, rec->height);
+	fprintf(stderr, "%s: mon: {MyDH: %d, MyDW: %d}\n", __func__,
+		m->virtual_scr.MyDisplayWidth, m->virtual_scr.MyDisplayHeight);
+#endif
+	if (m == NULL)
+		return (False);
+
+	//if (m->virtual_scr.CurrentDesk == desk &&
+	if (rec->x + (signed int)rec->width > 0 &&
+		(rec->x < 0 || rec->x <= m->virtual_scr.MyDisplayWidth) &&
 		rec->y + (signed int)rec->height > 0 &&
-		(rec->y < 0 || rec->y < Scr.MyDisplayHeight)) ?
-		True : False;
+		(rec->y < 0 || rec->y <= m->virtual_scr.MyDisplayHeight)) {
+			return (True);
+	}
+	return (False);
 }
 
 /* returns the FvwmWindow that contains the pointer or NULL if none */
@@ -589,16 +508,16 @@ Time get_server_time(void)
 
 void print_g(char *text, rectangle *g)
 {
-	fprintf(stderr,"%s: ", (text != NULL) ? text : "");
+	fvwm_debug(__func__, "%s: ", (text != NULL) ? text : "");
 	if (g == NULL)
 	{
-		fprintf(stderr, "(null)\n");
+		fvwm_debug(__func__, "(null)\n");
 
 		return;
 	}
-	fprintf(stderr,"%4d %4d %4dx%4d (%4d - %4d, %4d - %4d)\n",
-		g->x, g->y, g->width, g->height,
-		g->x, g->x + g->width, g->y, g->y + g->height);
+	fvwm_debug(__func__, "%4d %4d %4dx%4d (%4d - %4d, %4d - %4d)\n",
+		   g->x, g->y, g->width, g->height,
+		   g->x, g->x + g->width, g->y, g->y + g->height);
 
 	return;
 }
