@@ -10,8 +10,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 /* ---------------------------- included header files ---------------------- */
@@ -21,7 +20,6 @@
 #include <stdio.h>
 
 #include "libs/fvwmlib.h"
-#include "libs/FScreen.h"
 #include "libs/Grab.h"
 #include "libs/Parse.h"
 #include "libs/XResource.h"
@@ -42,6 +40,7 @@
 #include "ewmh.h"
 #include "icons.h"
 #include "add_window.h"
+#include "expand.h"
 
 /* ---------------------------- local definitions -------------------------- */
 
@@ -139,7 +138,7 @@ typedef struct
 	struct
 	{
 		preason_screen_t reason;
-		int screen;
+		char *screen;
 		rectangle g;
 		unsigned was_modified_by_ewmh_workingarea : 1;
 	} screen;
@@ -166,7 +165,7 @@ typedef struct
 	int desk;
 	int page_x;
 	int page_y;
-	int screen;
+	char *screen;
 } pl_start_style_t;
 
 typedef struct
@@ -359,7 +358,7 @@ static pl_penalty_t __pl_position_get_pos_simple(
 		fPointer = False;
 		ret_p->x = 0;
 		ret_p->y = 0;
-		n = GetMoveArguments(
+		n = GetMoveArguments(arg->place_fw,
 			&spos, arg->place_g.width, arg->place_g.height,
 			&ret_p->x, &ret_p->y, NULL, &fPointer, False);
 		spos = DEFAULT_PLACEMENT_POSITION_STRING;
@@ -553,8 +552,7 @@ static pl_penalty_t __pl_manual_get_pos_simple(
 		{
 			XMapRaised(dpy, Scr.SizeWindow);
 		}
-		FScreenGetScrRect(
-			NULL, FSCREEN_GLOBAL, &mx, &my, NULL, NULL);
+		FScreenGetScrRect(NULL, FSCREEN_GLOBAL, &mx, &my, NULL, NULL);
 		if (__move_loop(
 			    arg->exc, mx, my, DragWidth, DragHeight, &ret_p->x,
 			    &ret_p->y, False, CRS_POSITION))
@@ -607,7 +605,10 @@ static int __pl_minoverlap_get_next_x(const pl_arg_t *arg)
 	Bool rc;
 	int x;
 	int y;
+	struct monitor	*m = NULL;
 
+	m = (arg->place_fw && arg->place_fw->m) ?
+	    arg->place_fw->m : monitor_get_current();
 	x = arg->place_g.x;
 	y = arg->place_g.y;
 	if (arg->flags.use_percent == 1)
@@ -627,14 +628,14 @@ static int __pl_minoverlap_get_next_x(const pl_arg_t *arg)
 		xnew = xtest;
 	}
 	/* test the borders of the working area */
-	xtest = arg->page_p1.x + Scr.Desktops->ewmh_working_area.x;
+	xtest = arg->page_p1.x + m->Desktops->ewmh_working_area.x;
 	if (xtest > x)
 	{
 		xnew = MIN(xnew, xtest);
 	}
 	xtest = arg->page_p1.x +
-		(Scr.Desktops->ewmh_working_area.x +
-		 Scr.Desktops->ewmh_working_area.width) -
+		(m->Desktops->ewmh_working_area.x +
+		 m->Desktops->ewmh_working_area.width) -
 		arg->place_g.width;
 	if (xtest > x)
 	{
@@ -742,6 +743,7 @@ static int __pl_minoverlap_get_next_y(const pl_arg_t *arg)
 	int i;
 	rectangle g;
 	int y;
+	struct monitor	*mon;
 
 	y = arg->place_g.y;
 	if (arg->flags.use_percent == 1)
@@ -753,6 +755,9 @@ static int __pl_minoverlap_get_next_y(const pl_arg_t *arg)
 		start = CP_GET_NEXT_STEP;
 	}
 
+	mon = (arg->place_fw && arg->place_fw->m) ?
+	    arg->place_fw->m : monitor_get_current();
+
 	/* Test window at far bottom of screen */
 	ynew = arg->page_p2.y;
 	ytest = arg->page_p2.y - arg->place_g.height;
@@ -761,14 +766,14 @@ static int __pl_minoverlap_get_next_y(const pl_arg_t *arg)
 		ynew = ytest;
 	}
 	/* test the borders of the working area */
-	ytest = arg->page_p1.y + Scr.Desktops->ewmh_working_area.y;
+	ytest = arg->page_p1.y + mon->Desktops->ewmh_working_area.y;
 	if (ytest > y)
 	{
 		ynew = MIN(ynew, ytest);
 	}
 	ytest = arg->screen_g.y +
-		(Scr.Desktops->ewmh_working_area.y +
-		 Scr.Desktops->ewmh_working_area.height) -
+		(mon->Desktops->ewmh_working_area.y +
+		 mon->Desktops->ewmh_working_area.height) -
 		arg->place_g.height;
 	if (ytest > y)
 	{
@@ -1075,13 +1080,14 @@ static pl_penalty_t __pl_minoverlap_get_pos_penalty(
 	/* now handle the working area */
 	{
 		const pl_penalty_struct *mypp;
+		struct monitor *m = FindScreenOfXY(arg->place_g.x, arg->place_g.y);
 
 		mypp = (arg->scratch->pp != 0 && 0) ? arg->scratch->pp :
 			&arg->place_fw->pl_penalty;
 		if (arg->flags.use_ewmh_dynamic_working_areapercent == 1)
 		{
 			penalty += EWMH_STRUT_PLACEMENT_PENALTY(mypp) *
-				EWMH_GetStrutIntersection(
+				EWMH_GetStrutIntersection(m,
 					arg->place_g.x, arg->place_g.y,
 					arg->place_p2.x, arg->place_p2.y,
 					arg->flags.use_percent);
@@ -1092,7 +1098,7 @@ static pl_penalty_t __pl_minoverlap_get_pos_penalty(
 			 */
 			penalty +=
 				EWMH_STRUT_PLACEMENT_PENALTY(mypp) *
-				EWMH_GetBaseStrutIntersection(
+				EWMH_GetBaseStrutIntersection(m,
 					arg->place_g.x, arg->place_g.y,
 					arg->place_p2.x, arg->place_p2.y,
 					arg->flags.use_percent);
@@ -1407,6 +1413,7 @@ static int __place_get_nowm_pos(
 {
 	FvwmWindow *fw = exc->w.fw;
 	size_borders b;
+	struct monitor	*m = (fw && fw->m) ? fw->m : monitor_get_current();
 
 	if (!win_opts->flags.do_override_ppos)
 	{
@@ -1418,8 +1425,6 @@ static int __place_get_nowm_pos(
 		SUSE_START_ON_SCREEN(&pstyle->flags) &&
 		flags.do_honor_starts_on_screen)
 	{
-		fscreen_scr_t mangle_screen;
-
 		/* If StartsOnScreen has been given for a window, translate its
 		 * USPosition so that it is relative to that particular screen.
 		 *  If we don't do this, then a geometry would completely
@@ -1436,16 +1441,18 @@ static int __place_get_nowm_pos(
 		 * 2: Do NOT specify a Xinerama screen (or specify it to be
 		 * 'g') and give the geometry hint in terms of the global
 		 * screen. */
-		mangle_screen = FScreenFetchMangledScreenFromUSPosHints(
+#if 0
+		int mangle_screen = FScreenFetchMangledScreenFromUSPosHints(
 			&(fw->hints));
-		if (mangle_screen != FSCREEN_GLOBAL)
+		if (mangle_screen != 0)
 		{
 			/* whoever set this hint knew exactly what he was
 			 * doing; so ignore the StartsOnScreen style */
 			flags.do_honor_starts_on_screen = 0;
 			reason->pos.reason = PR_POS_USPOS_OVERRIDE_SOS;
 		}
-		else if (attr_g->x + attr_g->width < screen_g.x ||
+#endif
+		if (attr_g->x + attr_g->width < screen_g.x ||
 			 attr_g->x >= screen_g.x + screen_g.width ||
 			 attr_g->y + attr_g->height < screen_g.y ||
 			 attr_g->y >= screen_g.y + screen_g.height)
@@ -1454,8 +1461,11 @@ static int __place_get_nowm_pos(
 			 * screen.  Let's assume the application specified
 			 * global coordinates and translate them to the screen.
 			 */
+			fscreen_scr_arg	 arg;
+			arg.mouse_ev = NULL;
+			arg.name = start_style.screen;
 			FScreenTranslateCoordinates(
-				NULL, start_style.screen, NULL, FSCREEN_GLOBAL,
+				&arg, FSCREEN_BY_NAME, NULL, FSCREEN_GLOBAL,
 				&attr_g->x, &attr_g->y);
 			reason->pos.do_adjust_off_screen = 1;
 		}
@@ -1494,9 +1504,9 @@ static int __place_get_nowm_pos(
 		 * then 2) readjust relative to the current page. */
 		if (attr_g->x < 0)
 		{
-			attr_g->x += Scr.MyDisplayWidth;
+			attr_g->x += m->virtual_scr.MyDisplayWidth;
 		}
-		attr_g->x %= Scr.MyDisplayWidth;
+		attr_g->x %= m->virtual_scr.MyDisplayWidth;
 		attr_g->x -= pdeltax;
 		/* Noticed a quirk here. With some apps (e.g., xman), we find
 		 * the placement has moved 1 pixel away from where we
@@ -1505,9 +1515,9 @@ static int __place_get_nowm_pos(
 		 * -borderwidth 100 */
 		if (attr_g->y < 0)
 		{
-			attr_g->y += Scr.MyDisplayHeight;
+			attr_g->y += m->virtual_scr.MyDisplayHeight;
 		}
-		attr_g->y %= Scr.MyDisplayHeight;
+		attr_g->y %= m->virtual_scr.MyDisplayHeight;
 		attr_g->y -= pdeltay;
 		if (attr_g->x != old_x || attr_g->y != old_y)
 		{
@@ -1648,14 +1658,13 @@ static int __place_window(
 			flags.do_honor_starts_on_page = 0;
 			reason->page.reason = PR_PAGE_IGNORE_INVALID;
 			reason->page.do_ignore_starts_on_page = 1;
-			fvwm_msg(
-				WARN, "__place_window",
-				"invalid style combination used: StartsOnPage"
-				"/StartsOnDesk and SkipMapping don't work with"
-				" ManualPlacement and TileManualPlacement."
-				" Putting window on current page, please use"
-				" another placement style or"
-				" ActivePlacementHonorsStartsOnPage.");
+			fvwm_debug(__func__,
+				   "invalid style combination used: StartsOnPage"
+				   "/StartsOnDesk and SkipMapping don't work with"
+				   " ManualPlacement and TileManualPlacement."
+				   " Putting window on current page, please use"
+				   " another placement style or"
+				   " ActivePlacementHonorsStartsOnPage.");
 		}
 	}
 	/* get the screen coordinates to place window on */
@@ -1663,29 +1672,52 @@ static int __place_window(
 	{
 		if (flags.do_honor_starts_on_screen)
 		{
-			/* use screen from style */
-			FScreenGetScrRect(
-				NULL, start_style.screen, &screen_g.x,
-				&screen_g.y, &screen_g.width,
-				&screen_g.height);
-			reason->screen.screen = start_style.screen;
+			fscreen_scr_arg	 arg;
+			arg.mouse_ev = NULL;
+
+			/* FIXME:  expand the screen name here.  It's possible
+			 * we have been specified as:
+			 *
+			 *     Style foo StartsOnScreen $$[screen.pointer]
+			 *
+			 * we need to double-up the variable expansion, as the
+			 * Style command will expand it first, and we want to
+			 * expand $[pointer.screen] here.
+			 */
+			char *sc = SGET_START_SCREEN(*pstyle);
+			char *e = expand_vars(sc, NULL, False, True, NULL, exc);
+
+			arg.name = e;
+
+			FScreenGetScrRect(&arg, FSCREEN_BY_NAME,
+				&screen_g.x, &screen_g.y,
+				&screen_g.width, &screen_g.height);
+
+			/* FIXME:  Set the monitor here, but make
+			 * UPDATE_FVWM_SCREEN aware of how to do this.
+			 *
+			 * This is necessary for StartsOnScreen
+			 */
+			fw->m = monitor_by_name(arg.name);
+			free(e);
 		}
 		else
 		{
 			/* use global screen */
-			FScreenGetScrRect(
-				NULL, FSCREEN_GLOBAL, &screen_g.x, &screen_g.y,
-				&screen_g.width, &screen_g.height);
-			reason->screen.screen = FSCREEN_GLOBAL;
+			screen_g.x = 0;
+			screen_g.y = 0;
+			screen_g.width = monitor_get_all_widths();
+			screen_g.height = monitor_get_all_heights();
+			reason->screen.screen = "global";
 		}
 	}
 	else
 	{
 		/* use current screen */
-		FScreenGetScrRect(
-			NULL, FSCREEN_CURRENT, &screen_g.x, &screen_g.y,
+		FScreenGetScrRect(NULL, FSCREEN_CURRENT,
+			&screen_g.x, &screen_g.y,
 			&screen_g.width, &screen_g.height);
-		reason->screen.screen = FSCREEN_CURRENT;
+		reason->screen.screen = "current";
 	}
 
 	if (SPLACEMENT_MODE(&pstyle->flags) != PLACE_MINOVERLAPPERCENT &&
@@ -1706,7 +1738,8 @@ static int __place_window(
 	/* Don't alter the existing desk location during Capture/Recapture.  */
 	if (!win_opts->flags.do_override_ppos)
 	{
-		fw->Desk = Scr.CurrentDesk;
+		struct monitor	*m = fw->m ? fw->m : monitor_get_current();
+		fw->Desk = m->virtual_scr.CurrentDesk;
 		reason->desk.reason = PR_DESK_CURRENT;
 	}
 	else
@@ -1715,7 +1748,8 @@ static int __place_window(
 	}
 	if (S_IS_STICKY_ACROSS_DESKS(SFC(pstyle->flags)))
 	{
-		fw->Desk = Scr.CurrentDesk;
+		struct monitor	*m = fw->m ? fw->m : monitor_get_current();
+		fw->Desk = m->virtual_scr.CurrentDesk;
 		reason->desk.reason = PR_DESK_STICKY;
 	}
 	else if (SUSE_START_ON_DESK(&pstyle->flags) && start_style.desk &&
@@ -1767,6 +1801,10 @@ static int __place_window(
 			{
 				if (FW_W(t) == FW_W_TRANSIENTFOR(fw))
 				{
+					if (monitor_mode == MONITOR_TRACKING_M &&
+					    (fw->m != t->m))
+						continue;
+
 					fw->Desk = t->Desk;
 					reason->desk.reason =
 						PR_DESK_TRANSIENT;
@@ -1775,6 +1813,7 @@ static int __place_window(
 			}
 		}
 
+#if 0
 		{
 			/* migo - I am not sure this block is ever needed */
 
@@ -1799,6 +1838,7 @@ static int __place_window(
 				}
 			}
 		}
+#endif
 	}
 	reason->desk.desk = fw->Desk;
 	/* I think it would be good to switch to the selected desk
@@ -1806,11 +1846,13 @@ static int __place_window(
 	/*  RBW - 11/02/1998  --  I dont. */
 	if (!win_opts->flags.do_override_ppos && !DO_NOT_SHOW_ON_MAP(fw))
 	{
-		if (Scr.CurrentDesk != fw->Desk)
+		struct monitor	*m = fw->m ? fw->m : monitor_get_current();
+
+		if (m->virtual_scr.CurrentDesk != fw->Desk)
 		{
 			reason->desk.do_switch_desk = 1;
 		}
-		goto_desk(fw->Desk);
+		goto_desk(fw->Desk, m);
 	}
 	/* Don't move viewport if SkipMapping, or if recapturing the window,
 	 * adjust the coordinates later. Otherwise, just switch to the target
@@ -1823,22 +1865,24 @@ static int __place_window(
 	{
 		if (start_style.page_x != 0 && start_style.page_y != 0)
 		{
+			struct monitor	*m = (fw && fw->m) ? fw->m :
+				monitor_get_current();
 			px = start_style.page_x - 1;
 			py = start_style.page_y - 1;
 			reason->page.reason = PR_PAGE_STYLE;
-			px *= Scr.MyDisplayWidth;
-			py *= Scr.MyDisplayHeight;
+			px *= m->virtual_scr.MyDisplayWidth;
+			py *= m->virtual_scr.MyDisplayHeight;
 			if (!win_opts->flags.do_override_ppos &&
 			    !DO_NOT_SHOW_ON_MAP(fw))
 			{
-				MoveViewport(px,py,True);
+				MoveViewport(m, px,py,True);
 				reason->page.do_switch_page = 1;
 			}
 			else if (flags.do_honor_starts_on_page)
 			{
 				/*  Save the delta from current page */
-				pdeltax = Scr.Vx - px;
-				pdeltay = Scr.Vy - py;
+				pdeltax = m->virtual_scr.Vx - px;
+				pdeltay = m->virtual_scr.Vy - py;
 				reason->page.do_honor_starts_on_page = 1;
 			}
 		}
@@ -1859,6 +1903,17 @@ static int __place_window(
 			exc, pstyle, attr_g, flags, screen_g, start_style,
 			mode, win_opts, reason, pdeltax, pdeltay);
 	}
+
+	/* Check the desk here. */
+	if (!SUSE_START_ON_DESK(&pstyle->flags) &&
+		((!Restarting && Scr.flags.are_windows_captured)) &&
+		mode != PLACE_AGAIN) {
+		struct monitor *mnew = FindScreenOfXY(attr_g->x, attr_g->y);
+		fw->m = mnew;
+		do_move_window_to_desk(fw, mnew->virtual_scr.CurrentDesk);
+		pstyle->flags.initial_placement_done = 1;
+	}
+
 	reason->pos.x = attr_g->x;
 	reason->pos.y = attr_g->y;
 
@@ -1920,8 +1975,7 @@ static void __place_handle_x_resources(
 	if (GetResourceString(db, "fvwmscreen", client_argv[0], &rm_value) &&
 	    rm_value.size != 0)
 	{
-		SSET_START_SCREEN(
-			*pstyle, FScreenGetScreenArgument(rm_value.addr, 'c'));
+		SSET_START_SCREEN(*pstyle, rm_value.addr);
 		reason->screen.reason = PR_SCREEN_X_RESOURCE_FVWMSCREEN;
 		reason->screen.screen = SGET_START_SCREEN(*pstyle);
 		pstyle->flags.use_start_on_screen = 1;
@@ -1972,7 +2026,6 @@ static void __explain_placement(FvwmWindow *fw, pl_reason_t *reason)
 	char explanation[2048];
 	char *r;
 	char *s;
-	char t[32];
 	int do_show_page;
 	int is_placed_by_algo;
 
@@ -2078,7 +2131,7 @@ static void __explain_placement(FvwmWindow *fw, pl_reason_t *reason)
 		s += strlen(s);
 	}
 	/* screen */
-	if (FScreenIsEnabled() == True || FScreenIsSLSEnabled() == True)
+	if (FScreenIsEnabled() == True)
 	{
 		switch (reason->screen.reason)
 		{
@@ -2098,11 +2151,11 @@ static void __explain_placement(FvwmWindow *fw, pl_reason_t *reason)
 			r = "bug";
 			break;
 		}
-		FScreenSpecToString(t, 32, reason->screen.screen);
 		sprintf(
 			s, "  screen: %s: %d %d %dx%d (%s)\n",
-			t, reason->screen.g.x, reason->screen.g.y,
-			reason->screen.g.width, reason->screen.g.height, r);
+			reason->screen.screen, reason->screen.g.x,
+			reason->screen.g.y, reason->screen.g.width,
+			reason->screen.g.height, r);
 		s += strlen(s);
 		if (reason->screen.was_modified_by_ewmh_workingarea == 1)
 		{
@@ -2231,9 +2284,8 @@ static void __explain_placement(FvwmWindow *fw, pl_reason_t *reason)
 		sprintf(s, "    (adjusted to force window on page)\n");
 		s += strlen(s);
 	}
-	fvwm_msg(
-		INFO, "__explain_placement", explanation, (int)FW_W(fw),
-		fw->name.name);
+	fvwm_debug(__func__, explanation, (int)FW_W(fw),
+		   fw->name.name);
 
 	return;
 }
@@ -2273,10 +2325,32 @@ Bool setup_window_placement(
 	start_style.desk = SGET_START_DESK(*pstyle);
 	start_style.page_x = SGET_START_PAGE_X(*pstyle);
 	start_style.page_y = SGET_START_PAGE_Y(*pstyle);
-	start_style.screen = SGET_START_SCREEN(*pstyle);
+
+	/* FIXME:  expand the screen name here.  It's possible
+	 * we have been specified as:
+	 *
+	 *     Style foo StartsOnScreen $$[screen.pointer]
+	 *
+	 * we need to double-up the variable expansion, as the
+	 * Style command will expand it first, and we want to
+	 * expand $[pointer.screen] here.
+	 */
+	char *sc = SGET_START_SCREEN(*pstyle);
+	char *e = NULL;
+
+	if (sc != NULL)
+		e = expand_vars(sc, NULL, False, True, NULL, exc);
+
+	start_style.screen = (e != NULL) ? fxstrdup(e) : fxstrdup("");
+	fvwm_debug(__func__, "Expanding screen from '%s' -> '%s'",
+			sc ? sc : "null", start_style.screen);
 	rc = __place_window(
 		exc, pstyle, attr_g, start_style, mode, win_opts, &reason);
+
 	exc_destroy_context(exc);
+	free(e);
+	free(start_style.screen);
+
 	if (Scr.bo.do_explain_window_placement == 1)
 	{
 		__explain_placement(fw, &reason);
@@ -2329,7 +2403,7 @@ void CMD_PlaceAgain(F_CMD_ARGS)
 		{
 			return;
 		}
-		fw->Desk = Scr.CurrentDesk;
+		fw->Desk = fw->m->virtual_scr.CurrentDesk;
 		get_icon_geometry(fw, &old_g);
 		SET_ICON_MOVED(fw, 0);
 		AutoPlaceIcon(fw, NULL, False);
