@@ -11,8 +11,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 #include "config.h"
@@ -30,7 +29,7 @@
 
 /* If non-zero we are already repeating a function, so don't record the
  * command again. */
-static unsigned int repeat_depth = 0;
+static int repeat_depth = 0;
 
 #if 0
 typedef struct
@@ -116,22 +115,13 @@ Bool set_repeat_data(void *data, repeat_t type, const func_t *builtin)
 			 * FUNC_DONT_REPEAT flag set. */
 			return True;
 		}
-		if (last.command_line)
-		{
-			free(last.command_line);
-		}
 		/* Store a backup. */
 		last.command_line = (char *)data;
 		/* Since we stored the pointer the caller must not free it. */
 		return False;
 	case REPEAT_MENU:
 	case REPEAT_POPUP:
-		if (last.menu_name)
-		{
-			free(last.menu_name);
-		}
 		last.menu_name = (char *)data;
-		/* Since we stored the pointer the caller must not free it. */
 		return False;
 	case REPEAT_PAGE:
 	case REPEAT_DESK:
