@@ -18,10 +18,11 @@
 #include <stdio.h>
 
 #include "libs/fvwmlib.h"
-#include "libs/queue.h"
+#include "libs/fqueue.h"
 #include "libs/charmap.h"
 #include "libs/wcontext.h"
 #include "libs/Parse.h"
+#include "libs/FEvent.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "colorset.h"
@@ -132,10 +133,9 @@ static void schedule(
 		return;
 	}
 	/* create the new object */
-	new_obj = (sq_object_type *)safemalloc(sizeof(sq_object_type));
-	memset(new_obj, 0, sizeof(sq_object_type));
+	new_obj = fxcalloc(1, sizeof(sq_object_type));
 	new_obj->window = window;
-	new_obj->command = safestrdup(command);
+	new_obj->command = fxstrdup(command);
 	new_obj->time_to_execute = time_to_execute;
 	new_obj->period = period; /* 0 if this is not a periodic command */
 	/* set the job group id */
@@ -195,7 +195,7 @@ static void execute_obj_func(void *object, void *args)
 	if (obj->period > 0)
 	{
 		/* This is a periodic function, so reschedule it. */
-		sq_object_type *new_obj = (sq_object_type *)safemalloc(sizeof(sq_object_type));
+		sq_object_type *new_obj = fxmalloc(sizeof(sq_object_type));
 		memcpy(new_obj, obj, sizeof(sq_object_type));
 		obj->command = NULL; /* new_obj->command now points to cmd. */
 		new_obj->time_to_execute = fev_get_evtime() + new_obj->period;
@@ -284,8 +284,8 @@ void CMD_Schedule(F_CMD_ARGS)
 	n = GetIntegerArguments(action, &action, &ms, 1);
 	if (n <= 0)
 	{
-		fvwm_msg(ERR, "CMD_Schedule",
-			 "Requires time to schedule as argument");
+		fvwm_debug(__func__,
+			   "Requires time to schedule as argument");
 		return;
 	}
 	if (ms < 0)
