@@ -28,11 +28,19 @@
  *
  */
 
-#ifndef _SCREEN_
-#define _SCREEN_
+#ifndef FVWM_SCREEN_H
+#define FVWM_SCREEN_H
+
+#include "config.h"
+
+#include "update.h"
+#include "style.h"
+#include "fvwm.h"
+#include "misc.h"
 
 #include "libs/flist.h"
 #include "libs/Bindings.h"
+#include "libs/PictureBase.h"
 
 #define SIZE_HINDENT 5
 #define SIZE_VINDENT 3
@@ -76,16 +84,6 @@ typedef struct
 	int alpha_percent;
 } FvwmAcs;
 
-typedef struct
-{
-	Window win;
-	int isMapped;
-	/* command which is executed when the pan frame is entered */
-	char *command;
-	/* command which is executed when the pan frame is left*/
-	char *command_leave;
-} PanFrame;
-
 typedef enum
 {
 	/* button types */
@@ -299,40 +297,12 @@ typedef struct FvwmDecor
 	} flags;
 } FvwmDecor;
 
-typedef struct DesktopsInfo
-{
-	int desk;
-	char *name;
-
-	struct
-	{
-		int x;
-		int y;
-		int width;
-		int height;
-	} ewmh_working_area;
-	struct
-	{
-		int x;
-		int y;
-		int width;
-		int height;
-	} ewmh_dyn_working_area;
-
-	struct DesktopsInfo *next;
-} DesktopsInfo;
-
 typedef struct ScreenInfo
 {
 	unsigned long screen;
 	Screen *pscreen;
 	/* number of screens on display */
 	int NumberOfScreens;
-	/* my copy of DisplayWidth(dpy, screen) */
-	int MyDisplayWidth;
-	/* my copy of DisplayHeight(dpy, screen) */
-	int MyDisplayHeight;
-
 	/* the head of the fvwm window list */
 	FvwmWindow FvwmRoot;
 	/* the root window */
@@ -344,11 +314,6 @@ typedef struct ScreenInfo
 
 	flist *FWScheduledForDestroy;
 
-	PanFrame PanFrameTop;
-	PanFrame PanFrameLeft;
-	PanFrame PanFrameRight;
-	PanFrame PanFrameBottom;
-
 	/*dark gray pattern for shaded out menu items*/
 	Pixmap gray_bitmap;
 	/* dark gray pattern for inactive borders */
@@ -431,10 +396,6 @@ typedef struct ScreenInfo
 	FvwmWindow *StolenFocusFvwmWin;
 	FvwmWindow *focus_in_pending_window;
 	FvwmWindow *focus_in_requested_window;
-	/* #pixels to scroll on screen edge */
-	int EdgeScrollX;
-	/* #pixels to scroll on screen edge */
-	int EdgeScrollY;
 	/* buttons to grab in click to focus mode */
 	unsigned short buttons2grab;
 	int NumBoxes;
@@ -442,21 +403,12 @@ typedef struct ScreenInfo
 	int cascade_x;
 	int cascade_y;
 	FvwmWindow *cascade_window;
-	/* Max location for top left of virt desk*/
-	int VxMax;
-	int VyMax;
-	/* Current loc for top left of virt desk */
-	int Vx;
-	int Vy;
-
 	/*Max button-click delay for Function built-in*/
 	int ClickTime;
 	/* resistance to scrolling in desktop */
 	int ScrollDelay;
 	int MoveThreshold;
 	int OpaqueSize;
-	/* The current desktop number */
-	int CurrentDesk;
 	/* colormap focus style */
 	int ColormapFocus;
 	/* Limit on colors used in pixmaps */
@@ -481,6 +433,7 @@ typedef struct ScreenInfo
 		unsigned do_raise_over_unmanaged : 1;
 		unsigned is_modality_evil : 1;
 		unsigned is_raise_hack_needed : 1;
+		unsigned do_debug_randr : 1;
 	} bo; /* bug workaround control options */
 	struct
 	{
@@ -515,9 +468,6 @@ typedef struct ScreenInfo
 		unsigned is_wire_frame_displayed : 1;
 	} flags;
 
-	/* info for some desktops; the first entries should be generic info
-	 * correct for any desktop not in the list */
-	DesktopsInfo *Desktops;
 	/* the window of desktop type if any */
 	FvwmWindow *EwmhDesktop;
 	struct
@@ -527,6 +477,24 @@ typedef struct ScreenInfo
 	} last_added_item;
 } ScreenInfo;
 
+#define UPDATE_FVWM_SCREEN(fw)						   \
+	do {								   \
+		rectangle g;						   \
+		struct monitor *mnew;					   \
+									   \
+		get_unshaded_geometry((fw), &g);			   \
+		mnew = FindScreenOfXY((fw)->g.frame.x, (fw)->g.frame.y);   \
+		/* Avoid unnecessary updates. */			   \
+		if (mnew == (fw)->m &&					   \
+		    (fw)->Desk == mnew->virtual_scr.CurrentDesk)	   \
+			break;						   \
+		(fw)->m_prev = (fw)->m;					   \
+		(fw)->m = mnew;						   \
+		(fw)->Desk = mnew->virtual_scr.CurrentDesk;		   \
+		desk_add_fw((fw));					   \
+		BroadcastConfig(M_CONFIGURE_WINDOW, (fw));		   \
+	} while(0)
+
 /* A macro to to simplify he "ewmh desktop code" */
 #define IS_EWMH_DESKTOP(win) \
 	(Scr.EwmhDesktop && win == Scr.EwmhDesktop->wins.client)
@@ -551,4 +519,4 @@ void AddToDecor(F_CMD_ARGS, FvwmDecor *decor);
 
 extern ScreenInfo Scr;
 
-#endif /* _SCREEN_ */
+#endif /* FVWM_SCREEN_H */
