@@ -1,7 +1,9 @@
 /* -*-c-*- */
 
-#ifndef _STACK_H
-#define _STACK_H
+#ifndef FVWM_STACK_H
+#define FVWM_STACK_H
+
+#include "fvwm.h"
 
 #define DEBUG_STACK_RING 1
 #ifdef DEBUG_STACK_RING
@@ -13,8 +15,8 @@ FvwmWindow *get_next_window_in_stack_ring(const FvwmWindow *t);
 FvwmWindow *get_prev_window_in_stack_ring(const FvwmWindow *t);
 FvwmWindow *get_transientfor_fvwmwindow(const FvwmWindow *t);
 Bool position_new_window_in_stack_ring(FvwmWindow *t, Bool do_lower);
-void RaiseWindow(FvwmWindow *t);
-void LowerWindow(FvwmWindow *t);
+void RaiseWindow(FvwmWindow *t, Bool is_client_request);
+void LowerWindow(FvwmWindow *t, Bool is_client_request);
 Bool HandleUnusualStackmodes(
 	unsigned int stack_mode, FvwmWindow *r, Window rw, FvwmWindow *sib,
 	Window sibw);
@@ -44,4 +46,4 @@ void mark_transient_subtree(
 	Bool use_window_group_hint);
 
 
-#endif /* _STACK_H */
+#endif /* FVWM_STACK_H */
