@@ -1,10 +1,10 @@
 /* -*-c-*- */
 
-#ifndef _STYLE_
-#define _STYLE_
+#ifndef FVWM_STYLE_H
+#define FVWM_STYLE_H
 
-/* The default title in case the user doesn't supply one. */
-#define DEFAULT_TITLE_FORMAT "%n"
+#include "fvwm.h"
+#include "update.h"
 
 /* access to the special flags of a style */
 /* call these with a pointer to a style_flags struct */
@@ -677,4 +677,4 @@ void free_icon_boxes(icon_boxes *ib);
 void style_destroy_style(style_id_t s_id);
 void print_styles(int verbose);
 
-#endif /* _STYLE_ */
+#endif /* FVWM_STYLE_H */
