@@ -25,7 +25,6 @@
 #include <limits.h>
 
 #include "libs/fvwmlib.h"
-#include "libs/FScreen.h"
 #include "libs/FGettext.h"
 #include "libs/Parse.h"
 #include "libs/Strings.h"
@@ -74,17 +73,20 @@ static char *get_desk_title(int desk, unsigned long flags, Bool is_top_title)
 {
 	char *desk_name;
 	char *tlabel;
+	struct monitor	*m = monitor_get_current();
 
-	desk_name = GetDesktopName(desk);
+	desk_name = GetDesktopName(m, desk);
 	if (desk_name != NULL)
 	{
-		tlabel = (char *)safemalloc(strlen(desk_name)+50);
+		tlabel = fxmalloc(strlen(desk_name)+50);
 	}
 	else
 	{
-		tlabel = (char *)safemalloc(50);
+		tlabel = fxmalloc(50);
 	}
 
+	/* TA:  FIXME! xasprintf() */
+
 	if (desk_name != NULL)
 	{
 		if (flags & NO_NUM_IN_DESK_TITLE)
@@ -214,7 +216,8 @@ void CMD_WindowList(F_CMD_ARGS)
 	char scut = '0';              /* Current short cut key */
 	char *opts=NULL;
 	char *tok=NULL;
-	int desk = Scr.CurrentDesk;
+	struct monitor	*mon = monitor_get_current();
+	int desk = mon->virtual_scr.CurrentDesk;
 	unsigned long flags = SHOW_DEFAULT;
 	char *func = NULL;
 	char *ffunc = NULL;
@@ -300,7 +303,7 @@ void CMD_WindowList(F_CMD_ARGS)
 			}
 			else if (StrEquals(tok,"CurrentDesk"))
 			{
-				desk = Scr.CurrentDesk;
+				desk = mon->virtual_scr.CurrentDesk;
 				flags &= ~SHOW_ALLDESKS;
 			}
 			else if (StrEquals(tok,"NotAlphabetic"))
@@ -436,7 +439,8 @@ void CMD_WindowList(F_CMD_ARGS)
 			else if (StrEquals(tok,"UseListSkip"))
 			{
 				/* deprecated as of 02-May-2007 (SS) */
-				fprintf(stderr, "UseListSkip is deprecated. Please use \"UseSkipList\".\n");
+				fvwm_debug(__func__,
+					   "UseListSkip is deprecated. Please use \"UseSkipList\".\n");
 				skiplist_mode = 1;
 			}
 			else if (StrEquals(tok,"UseSkipList"))
@@ -446,7 +450,8 @@ void CMD_WindowList(F_CMD_ARGS)
 			else if (StrEquals(tok,"OnlyListSkip"))
 			{
 				/* deprecated as of 02-May-2007 (SS) */
-				fprintf(stderr, "OnlyListSkip is deprecated. Please use \"OnlySkipList\".\n");
+				fvwm_debug(__func__,
+					   "OnlyListSkip is deprecated. Please use \"OnlySkipList\".\n");
 				skiplist_mode = 2;
 			}
 			else if (StrEquals(tok,"OnlySkipList"))
@@ -552,13 +557,12 @@ void CMD_WindowList(F_CMD_ARGS)
 			}
 			else if (!opts || !*opts)
 			{
-				default_action = safestrdup(tok);
+				default_action = fxstrdup(tok);
 			}
 			else
 			{
-				fvwm_msg(
-					ERR, "WindowList","Unknown option '%s'",
-					tok);
+				fvwm_debug(__func__, "Unknown option '%s'",
+					   tok);
 			}
 			if (tok)
 			{
@@ -853,7 +857,7 @@ void CMD_WindowList(F_CMD_ARGS)
 				free_name = True;
 			}
 
-			t_hot = safemalloc(strlen(name) + 80);
+			t_hot = fxmalloc(strlen(name) + 80);
 			if (use_hotkey)
 			{
 				/* Generate label */
@@ -923,34 +927,23 @@ void CMD_WindowList(F_CMD_ARGS)
 				}
 				if (flags & SHOW_SCREEN)
 				{
-					fscreen_scr_arg fscr;
-					int scr;
-
-					fscr.xypos.x =
-						Scr.Vx + t->g.frame.x +
-						t->g.frame.width / 2;
-					fscr.xypos.y =
-						Scr.Vy + t->g.frame.y +
-						t->g.frame.height / 2;
-					scr = FScreenGetScrId(
-						&fscr, FSCREEN_XYPOS);
-					sprintf(loc, "@%d", scr);
+					sprintf(loc, "@%s", t->m->si->name);
 					strcat(tname, loc);
 				}
 				if (flags & SHOW_PAGE_X)
 				{
 					sprintf(loc, "+%d",
-						(Scr.Vx + t->g.frame.x +
+						(mon->virtual_scr.Vx + t->g.frame.x +
 						 t->g.frame.width / 2) /
-						Scr.MyDisplayWidth);
+						mon->virtual_scr.MyDisplayWidth);
 					strcat(tname, loc);
 				}
 				if (flags & SHOW_PAGE_Y)
 				{
 					sprintf(loc, "+%d",
-						(Scr.Vy + t->g.frame.y +
+						(mon->virtual_scr.Vy + t->g.frame.y +
 						 t->g.frame.height/2) /
-						Scr.MyDisplayHeight);
+						mon->virtual_scr.MyDisplayHeight);
 					strcat(tname, loc);
 				}
 				if (!(flags & NO_LAYER))
@@ -1007,9 +1000,11 @@ void CMD_WindowList(F_CMD_ARGS)
 				strcat(t_hot,tname);
 			}
 			ffunc = func ? func : "WindowListFunc";
-			tfunc = safemalloc(strlen(ffunc) + 36);
+			tfunc = fxmalloc(strlen(ffunc) + 36);
 			/* support two ways for now: window context
 			 * (new) and window id param (old) */
+
+			/* TA:  FIXME!  xasprintf() */
 			sprintf(tfunc, "WindowId %lu %s %lu",
 				FW_W(t), ffunc, FW_W(t));
 			AddToMenu(
