@@ -1,7 +1,7 @@
 /* -*-c-*- */
 
-#ifndef FFT_H
-#define FFT_H
+#ifndef FVWMLIB_FFT_H
+#define FVWMLIB_FFT_H
 
 /*
  * Note: This "warper" is far from being complete! if you need something
@@ -12,15 +12,9 @@
 
 /* ---------------------------- included header files ---------------------- */
 
-/* no compat to avoid problems in the future */
-#define _XFT_NO_COMPAT_ 1
-
-#ifdef HAVE_XFT
-#define Picture XRenderPicture
-#include <X11/Xft/Xft.h>
-#undef Picture
-#endif
+#include "config.h"
 
+#include "fvwm_x11.h"
 #include "FRender.h"
 
 /* ---------------------------- global definitions ------------------------- */
@@ -680,4 +674,4 @@ void FftPDumyFunc(void);
 
 #endif
 
-#endif /* FFT_H */
+#endif /* FVWMLIB_FFT_H */
