@@ -17,36 +17,16 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
-/*
- * This module is based on FvwmModuleDebugger which has the following
- * copyright:
- *
- * This module, and the entire ModuleDebugger program, and the concept for
- * interfacing this module to the Window Manager, are all original work
- * by Robert Nation
- *
- * Copyright 1994, Robert Nation. No guarantees or warantees or anything
- * are provided or implied in any way whatsoever. Use this program at your
- * own risk. Permission to use this program for any purpose is given,
- * as long as the copyright is kept intact.
- */
+/* This module is based on FvwmModuleDebugger */
 
 /* ---------------------------- included header files ----------------------- */
 
 #include "config.h"
 
 #include <stdio.h>
-/*
- * rplay includes:
- */
-#ifdef HAVE_RPLAY
-#include <rplay.h>
-#undef M_ERROR /* Solaris fix */
-#endif
 
 #include "libs/Module.h"
 #include "libs/fvwmlib.h"
@@ -64,11 +44,10 @@
 
 /* ---------------------------- local definitions --------------------------- */
 
-#define BUILTIN_STARTUP         (MAX_TOTAL_MESSAGES)
-#define BUILTIN_SHUTDOWN        (MAX_TOTAL_MESSAGES + 1)
-#define BUILTIN_UNKNOWN         (MAX_TOTAL_MESSAGES + 2)
+#define BUILTIN_STARTUP         0
+#define BUILTIN_SHUTDOWN        1
+#define BUILTIN_UNKNOWN         2
 #define MAX_BUILTIN             3
-#define MAX_RPLAY_HOSTNAME_LEN  MAX_MODULE_INPUT_TEXT_LEN
 #define SYNC_MASK_M             (M_DESTROY_WINDOW | M_LOWER_WINDOW | \
 	M_RESTACK | M_CONFIGURE_WINDOW)
 #define SYNC_MASK_MX            (M_EXTENDED_MSG)
@@ -85,11 +64,15 @@ typedef struct
 {
 	char *name;
 	int action_arg;
+	union
+	{
+		char *action;
+	} action;
 } event_entry;
 
 /* ---------------------------- forward declarations ------------------------ */
 
-void execute_event(short, unsigned long*);
+void execute_event(event_entry*, short, unsigned long*);
 void config(void);
 RETSIGTYPE DeadPipe(int);
 static RETSIGTYPE TerminateHandler(int);
@@ -112,65 +95,87 @@ static Bool audio_compat = False;
 static char *audio_play_dir = NULL;
 
 #define ARG_NO_WINID 1024  /* just a large number */
+#define ARG_EXPECTS_CHAR 1025
 
-static event_entry event_table[MAX_TOTAL_MESSAGES+MAX_BUILTIN] =
+#define EVENT_ENTRY(name,action_arg) { name, action_arg, {NULL} }
+static event_entry message_event_table[] =
 {
-	{ "new_page", -1 },
-	{ "new_desk", 0 | ARG_NO_WINID },
-	{ "old_add_window", 0 },
-	{ "raise_window", 0 },
-	{ "lower_window", 0 },
-	{ "old_configure_window", 0 },
-	{ "focus_change", 0 },
-	{ "destroy_window", 0 },
-	{ "iconify", 0 },
-	{ "deiconify", 0 },
-	{ "window_name", 0 },
-	{ "icon_name", 0 },
-	{ "res_class", 0 },
-	{ "res_name", 0 },
-	{ "end_windowlist", -1 },
-	{ "icon_location", 0 },
-	{ "map", 0 },
-	{ "error", -1 },
-	{ "config_info", -1 },
-	{ "end_config_info", -1 },
-	{ "icon_file", 0 },
-	{ "default_icon", -1 },
-	{ "string", -1 },
-	{ "mini_icon", 0 },
-	{ "windowshade", 0 },
-	{ "dewindowshade", 0 },
-	{ "visible_name", 0 },
-	{ "sendconfig", -1 },
-	{ "restack", -1 },
-	{ "add_window", 0 },
-	{ "configure_window", 0 },
-	/* begin of extended message area */
-	{ "visible_icon_name", 0 },
-	{ "enter_window", 0 },
-	{ "leave_window", 0 },
-	{ "property_change", 0},
-	/* built in events */
-#ifdef M_BELL
-	{ "beep", -1 },
-#endif
-#ifdef M_TOGGLEPAGE
-	{ "toggle_paging", -1 },
-#endif
-	/* add builtins here */
-	{ "startup", -1 },
-	{ "shutdown", -1 },
-	{ "unknown", -1}
+	EVENT_ENTRY( "new_page", -1 ),
+	EVENT_ENTRY( "new_desk", 0 | ARG_NO_WINID ),
+	EVENT_ENTRY( "old_add_window", 0 ),
+	EVENT_ENTRY( "raise_window", 0 ),
+	EVENT_ENTRY( "lower_window", 0 ),
+	EVENT_ENTRY( "old_configure_window", 0 ),
+	EVENT_ENTRY( "focus_change", 0 ),
+	EVENT_ENTRY( "destroy_window", 0 ),
+	EVENT_ENTRY( "iconify", 0 ),
+	EVENT_ENTRY( "deiconify", 0 ),
+	EVENT_ENTRY( "window_name", 0 ),
+	EVENT_ENTRY( "icon_name", 0 ),
+	EVENT_ENTRY( "res_class", 0 ),
+	EVENT_ENTRY( "res_name", 0 ),
+	EVENT_ENTRY( "end_windowlist", -1 ),
+	EVENT_ENTRY( "icon_location", 0 ),
+	EVENT_ENTRY( "map", 0 ),
+	EVENT_ENTRY( "error", -1 ),
+	EVENT_ENTRY( "config_info", -1 ),
+	EVENT_ENTRY( "end_config_info", -1 ),
+	EVENT_ENTRY( "icon_file", 0 ),
+	EVENT_ENTRY( "default_icon", -1 ),
+	EVENT_ENTRY( "string", -1 ),
+	EVENT_ENTRY( "mini_icon", 0 ),
+	EVENT_ENTRY( "windowshade", 0 ),
+	EVENT_ENTRY( "dewindowshade", 0 ),
+	EVENT_ENTRY( "visible_name", 0 ),
+	EVENT_ENTRY( "sendconfig", -1 ),
+	EVENT_ENTRY( "restack", -1 ),
+	EVENT_ENTRY( "add_window", 0 ),
+	EVENT_ENTRY( "configure_window", 0 ),
+	EVENT_ENTRY(NULL,0)
+};
+static event_entry extended_message_event_table[] =
+{
+	EVENT_ENTRY( "visible_icon_name", 0 ),
+	EVENT_ENTRY( "enter_window", 0 ),
+	EVENT_ENTRY( "leave_window", 0 ),
+	EVENT_ENTRY( "property_change", 0),
+	EVENT_ENTRY( "monitor_enabled", 0 | ARG_EXPECTS_CHAR),
+	EVENT_ENTRY( "monitor_disabled", 0 | ARG_EXPECTS_CHAR),
+	EVENT_ENTRY( "monitor_changed", 0 | ARG_EXPECTS_CHAR),
+	EVENT_ENTRY( "monitor_focus", 0 | ARG_EXPECTS_CHAR),
+	EVENT_ENTRY( "echo", 0 | ARG_EXPECTS_CHAR),
+	EVENT_ENTRY( "reply", 0), /* FvwmEvent will never receive MX_REPLY */
+	EVENT_ENTRY(NULL,0)
+};
+static event_entry builtin_event_table[] =
+{
+	EVENT_ENTRY( "startup", -1 ),
+	EVENT_ENTRY( "shutdown", -1 ),
+	EVENT_ENTRY( "unknown", -1),
+	EVENT_ENTRY(NULL,0)
 };
 
-/* define the action table  */
-static char *action_table[MAX_TOTAL_MESSAGES+MAX_BUILTIN];
-
-#ifdef HAVE_RPLAY
-static int rplay_fd = -1;
-static RPLAY *rplay_table[MAX_TOTAL_MESSAGES+MAX_BUILTIN];
+#if 0
+/* These are some events described in the man page, which does not exist in
+   fvwm. */
+static event_entry future_event_table[] =
+{
+#ifdef M_BELL
+	EVENT_ENTRY( "beep", -1 ),
 #endif
+	EVENT_ENTRY(NULL,0)
+}
+#endif
+
+static event_entry* event_tables[] =
+{
+	message_event_table,
+	extended_message_event_table,
+	builtin_event_table,
+	NULL
+};
+
+#undef EVENT_ENTRY
 
 static unsigned int m_selected = 0;
 static unsigned int mx_selected = 0;
@@ -210,7 +215,7 @@ int main(int argc, char **argv)
 	int total, remaining, count, event;
 	int is_extended_msg;
 
-	cmd_line = (char *)safemalloc(1);
+	cmd_line = fxmalloc(1);
 	*cmd_line = 0;
 	/* Save our program  name - for error events */
 	if ((s=strrchr(argv[0], '/')))
@@ -239,7 +244,7 @@ int main(int argc, char **argv)
 	/* account for '*' */
 	MyNameLen=strlen(s)+1;
 	/* account for \0 */
-	MyName = safemalloc(MyNameLen+1);
+	MyName = fxmalloc(MyNameLen+1);
 	*MyName='*';
 	/* append name */
 	strcpy(MyName+1, s);
@@ -253,7 +258,7 @@ int main(int argc, char **argv)
 	if ((argc != 6)&&(argc != 7))
 	{
 		fprintf(stderr, "%s Version "VERSION" should only be "
-			"executed by fvwm!\n", MyName+1);
+			   "executed by fvwm!\n", MyName+1);
 		exit(1);
 	}
 
@@ -286,7 +291,7 @@ int main(int argc, char **argv)
 	/* configure events */
 	config();
 	/* Startup event */
-	execute_event(BUILTIN_STARTUP, NULL);
+	execute_event(builtin_event_table, BUILTIN_STARTUP, NULL);
 	if (start_audio_delay)
 	{
 		last_time = time(0);
@@ -314,6 +319,7 @@ int main(int argc, char **argv)
 	while (!isTerminated)
 	{
 		unsigned long msg_bit;
+		event_entry *event_table;
 
 		if ((count = read(fd[1], header, FvwmPacketHeaderSize_byte)) <=
 		    0)
@@ -373,26 +379,26 @@ int main(int argc, char **argv)
 			event++;
 			msg_bit >>= 1;
 		}
-		if (event == -1)
-		{
-			event = BUILTIN_UNKNOWN;
-		}
-		else if (event >= MAX_MESSAGES && !is_extended_msg)
-		{
-			event = BUILTIN_UNKNOWN;
-		}
-		else if (event >= MAX_EXTENDED_MESSAGES && is_extended_msg)
+		if (
+			event == -1 ||
+			(event >= MAX_MESSAGES && !is_extended_msg) ||
+			(event >= MAX_EXTENDED_MESSAGES && is_extended_msg))
 		{
+			event_table = builtin_event_table;
 			event = BUILTIN_UNKNOWN;
 		}
 		else if (is_extended_msg)
 		{
-			event += MAX_MESSAGES;
+			event_table = extended_message_event_table;
 		}
-		execute_event(event, body);
+		else
+		{
+			event_table = message_event_table;
+		}
+		execute_event(event_table, event, body);
 		unlock_event(header[1]);
 	} /* while */
-	execute_event(BUILTIN_SHUTDOWN, NULL);
+	execute_event(builtin_event_table, BUILTIN_SHUTDOWN, NULL);
 
 	return 0;
 }
@@ -405,49 +411,34 @@ int main(int argc, char **argv)
  *    execute_event - actually executes the actions from lookup table
  *
  */
-void execute_event(short event, unsigned long *body)
+void execute_event(event_entry *event_table, short event, unsigned long *body)
 {
-#ifdef HAVE_RPLAY
-	/* this is the sign that rplay is used */
-	if (rplay_fd != -1 && rplay_table[event])
-	{
-		if (rplay(rplay_fd, rplay_table[event]) >= 0)
-		{
-			last_time = now;
-		}
-		else
-		{
-			rplay_perror("rplay");
-		}
-		return;
-	}
-#endif
-
-	if (action_table[event])
+	if (event_table[event].action.action != NULL)
 	{
 		char *buf = NULL;
 		int len = 0;
+		char *action;
 
-		len = strlen(cmd_line) + strlen(action_table[event]) + 32;
+		action = event_table[event].action.action;
+		len = strlen(cmd_line) + strlen(action) + 32;
 		if (audio_play_dir)
 		{
 			len += strlen(audio_play_dir);
 		}
-		buf = (char *)safemalloc(len);
+		buf = fxmalloc(len);
 		if (audio_compat)
 		{
 			/* Don't use audio_play_dir if it's NULL or if
 			 * the sound file is an absolute pathname. */
 			if (!audio_play_dir || audio_play_dir[0] == '\0' ||
-			    action_table[event][0] == '/')
+			    action[0] == '/')
 			{
-				sprintf(buf,"%s %s", cmd_line,
-					action_table[event]);
+				sprintf(buf,"%s %s", cmd_line, action);
 			}
 			else
 			{
 				sprintf(buf,"%s %s/%s &", cmd_line,
-					audio_play_dir, action_table[event]);
+					audio_play_dir, action);
 			}
 			if (!system(buf))
 			{
@@ -470,20 +461,22 @@ void execute_event(short event, unsigned long *body)
 				{
 					action_arg &= ~ARG_NO_WINID;
 					sprintf(buf, "%s %s %ld", cmd_line,
-						action_table[event],
-						body[action_arg]);
+						action,	body[action_arg]);
 				}
 				else
 				{
 					sprintf(buf, "%s %s 0x%lx", cmd_line,
-						action_table[event],
-						body[action_arg]);
+						action,	body[action_arg]);
 				}
 			}
+			/* monitor_* events. */
+			else if (action_arg != -1 && (action_arg & ARG_EXPECTS_CHAR)) {
+				sprintf(buf, "%s %s %s", cmd_line, action,
+						(char *)(&body[3]));
+			}
 			else
 			{
-				sprintf(buf,"%s %s", cmd_line,
-					action_table[event]);
+				sprintf(buf,"%s %s", cmd_line, action);
 			}
 			/* let fvwm execute the function */
 			SendText(fd, buf, fw);
@@ -505,20 +498,7 @@ void execute_event(short event, unsigned long *body)
  *
  */
 
-char *table[]=
-{
-	"Cmd",
-	"Delay",
-	"Dir",
-	"PassID",
-	"PlayCmd",
-	"RplayHost",
-	"RplayPriority",
-	"RplayVolume",
-	"StartDelay"
-}; /* define entries here, if this list becomes unsorted, use FindToken */
-
-void handle_config_line(char *buf, char **phost)
+void handle_config_line(char *buf)
 {
 	char *event;
 	char *action;
@@ -527,10 +507,16 @@ void handle_config_line(char *buf, char **phost)
 	char **e;
 	int i;
 	int found;
-#ifdef HAVE_RPLAY
-	int volume   = RPLAY_DEFAULT_VOLUME;
-	int priority = RPLAY_DEFAULT_PRIORITY;
-#endif
+
+	char *table[]=
+	{
+		"Cmd",
+		"Delay",
+		"Dir",
+		"PassID",
+		"StartDelay"
+	}; /* define entries here, if this list becomes unsorted, use FindToken */
+
 
 	p = buf + MyNameLen;
 	/* config option ? */
@@ -541,17 +527,6 @@ void handle_config_line(char *buf, char **phost)
 		token = PeekToken(p, &p);
 		switch (e - (char**)table)
 		{
-		case 4:
-			/* PlayCmd */
-			if (!audio_compat)
-			{
-				/* PlayCmd */
-				fprintf(stderr,
-					"%s: PlayCmd supported only when"
-					" invoked as FvwmAudio\n", MyName+1);
-				break;
-			}
-			/* fall through */
 		case 0:
 			/* Cmd */
 			if (cmd_line)
@@ -560,11 +535,11 @@ void handle_config_line(char *buf, char **phost)
 			}
 			if (token)
 			{
-				cmd_line = safestrdup(token);
+				cmd_line = fxstrdup(token);
 			}
 			else
 			{
-				cmd_line = safestrdup("");
+				cmd_line = fxstrdup("");
 			}
 			break;
 
@@ -580,9 +555,9 @@ void handle_config_line(char *buf, char **phost)
 			/* Dir */
 			if (!audio_compat)
 			{
-				fprintf(stderr,
-					"%s: Dir supported only when invoked as"
-					" FvwmAudio\n", MyName+1);
+				fvwm_debug(__func__,
+					   "%s: Dir supported only when invoked as"
+					   " FvwmAudio\n", MyName+1);
 				break;
 			}
 			if (token)
@@ -591,7 +566,7 @@ void handle_config_line(char *buf, char **phost)
 				{
 					free(audio_play_dir);
 				}
-				audio_play_dir = safestrdup(token);
+				audio_play_dir = fxstrdup(token);
 			}
 			break;
 
@@ -599,53 +574,7 @@ void handle_config_line(char *buf, char **phost)
 			/* PassID */
 			PassID = True;
 			break;
-
-#ifdef HAVE_RPLAY
-		case 5:
-			/* RPlayHost */
-			if (*phost)
-			{
-				free(*phost);
-				*phost = NULL;
-			}
-			if (token && (*token == '$'))
-			{
-				/* Check for $HOSTDISPLAY */
-				char *c1 = (char *)getenv(token + 1);
-				char *c2;
-
-				*phost = safestrdup(c1);
-				c2 = *phost;
-				while (c1 && *c1 != ':')
-				{
-					*c2++ = *c1++;
-				}
-				*c2 = '\0';
-			}
-			else
-			{
-				*phost = safestrdup(token);
-			}
-			break;
-
-		case 6:
-			/* RplayPriority */
-			if (token)
-			{
-				priority = atoi(token);
-			}
-			break;
-
-		case 7:
-			/* RplayVolume */
-			if (token)
-			{
-				volume = atoi(token);
-			}
-			break;
-#endif
-
-		case 8:
+		case 4:
 			/* StartDelay */
 			if (token)
 			{
@@ -657,6 +586,8 @@ void handle_config_line(char *buf, char **phost)
 	}
 	else
 	{
+		event_entry **event_table;
+
 		/* test for isspace(*p) ??? */
 		p = GetNextSimpleOption(p, &event);
 		p = GetNextSimpleOption(p, &action);
@@ -671,42 +602,45 @@ void handle_config_line(char *buf, char **phost)
 				free(action);
 			}
 			fprintf(stderr, "%s: incomplete event definition %s\n",
-				MyName + 1, buf);
+				   MyName + 1, buf);
 			return;
 		}
-		for (found = 0, i = 0;
-		     !found && i < MAX_TOTAL_MESSAGES + MAX_BUILTIN; i++)
+		event_table = event_tables;
+		found = 0;
+		while(found == 0 && *event_table != NULL)
 		{
-			if (MatchToken(event, event_table[i].name))
+			event_entry *loop_event;
+
+			for (loop_event = *event_table, i = 0;
+			     found == 0 && loop_event->name != NULL;
+			     loop_event++, i++)
 			{
-#ifdef HAVE_RPLAY
-				rplay_table[i] = rplay_create(RPLAY_PLAY);
-				rplay_set(
-					rplay_table[i], RPLAY_APPEND,
-					RPLAY_SOUND, action, RPLAY_PRIORITY,
-					priority, RPLAY_VOLUME, volume, NULL);
-#endif
-				if (action_table[i])
+				if (MatchToken(event, loop_event->name))
 				{
-					free(action_table[i]);
-				}
-				action_table[i] = action;
-				found = 1;
-				if (i < MAX_MESSAGES)
-				{
-					m_selected |= (1 << i);
-				}
-				else if (i < MAX_TOTAL_MESSAGES)
-				{
-					mx_selected |=
-						(1 << (i - MAX_MESSAGES));
+					if (loop_event->action.action != NULL)
+					{
+						free(loop_event->action.action);
+					}
+					loop_event->action.action = action;
+					found = 1;
+					if (*event_table == message_event_table)
+					{
+						m_selected |= (1 << i);
+					}
+					else if (
+						*event_table ==
+						extended_message_event_table)
+					{
+						mx_selected |= (1 << i);
+					}
 				}
 			}
+			event_table++;
 		}
 		if (!found)
 		{
 			fprintf(stderr, "%s: unknown event type: %s\n",
-				MyName+1, event);
+				   MyName+1, event);
 			if (action)
 			{
 				free(action);
@@ -724,21 +658,6 @@ void handle_config_line(char *buf, char **phost)
 void config(void)
 {
 	char *buf;
-	int i;
-	char *host = NULL;
-
-#ifdef HAVE_RPLAY
-	host = safestrdup(rplay_default_host());
-#endif
-
-	/* Intialize all the actions */
-	for (i = 0; i < MAX_TOTAL_MESSAGES+MAX_BUILTIN; i++)
-	{
-		action_table[i] = NULL;
-#ifdef HAVE_RPLAY
-		rplay_table[i] = NULL;
-#endif
-	}
 
 	/* get config lines with my name */
 	InitGetConfigLine(fd,MyName);
@@ -753,24 +672,9 @@ void config(void)
 		/* Search for MyName (normally *FvwmEvent) */
 		if (strncasecmp(buf, MyName, MyNameLen) == 0)
 		{
-			handle_config_line(buf, &host);
+			handle_config_line(buf);
 		}
 	}
-
-#ifdef HAVE_RPLAY
-	/* Builtin rplay support is enabled when
-	 * FvwmAudioPlayCmd == builtin-rplay. */
-	if (StrEquals(cmd_line, "builtin-rplay"))
-	{
-		if ((rplay_fd = rplay_open(host)) < 0)
-		{
-			rplay_perror("rplay_open");
-			exit(1);
-		}
-	}
-#endif
-
-	return;
 }
 
 /*
@@ -795,7 +699,7 @@ TerminateHandler(int nonsense)
  */
 RETSIGTYPE DeadPipe(int flag)
 {
-	execute_event(BUILTIN_SHUTDOWN, NULL);
+	execute_event(builtin_event_table, BUILTIN_SHUTDOWN, NULL);
 	exit(flag);
 	SIGNAL_RETURN;
 }
