@@ -42,19 +42,14 @@
 #include "config.h"
 #include "libs/fvwmlib.h"
 #include "libs/Colorset.h"
+#include "libs/FEvent.h"
 
 #include <stdio.h>
 #include <ctype.h>
 
-#include <X11/Xlib.h>
-#include <X11/X.h>
-#include <X11/Xutil.h>
-#include <X11/cursorfont.h>
-#define XK_MISCELLANY
-#include <X11/keysymdef.h>
-#include <X11/Xatom.h>                  /* for XA_CUT_BUFFER0 */
+// #define XK_MISCELLANY
 
-#include <FvwmForm.h>
+#include "FvwmForm.h"
 
 static void process_regular_char_input(unsigned char *buf);
 static int process_tabtypes(unsigned char * buf);
@@ -569,7 +564,7 @@ static void process_history(int direction) {
     }
   }
   CF.cur_input->input.value =
-    safestrdup(CF.cur_input->input.value_history_ptr[count]);
+    fxstrdup(CF.cur_input->input.value_history_ptr[count]);
   CF.cur_input->input.n = strlen(CF.cur_input->input.value);
   CF.cur_input->input.value_history_yankat = count; /* new yank point */
   CF.cur_input->input.buf = CF.cur_input->input.n;
@@ -638,8 +633,8 @@ static void process_regular_char_input(unsigned char *buf) {
   if (++(CF.cur_input->input.n) >= CF.cur_input->input.buf) {
     CF.cur_input->input.buf += CF.cur_input->input.size;
     CF.cur_input->input.value =
-      (char *)saferealloc(CF.cur_input->input.value,
-			  CF.cur_input->input.buf);
+      fxrealloc(CF.cur_input->input.value, CF.cur_input->input.buf,
+		      sizeof(CF.cur_input->input.value));
   }
   dp = CF.cur_input->input.value + CF.cur_input->input.n;
   sp = dp - 1;
