@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 /*
@@ -30,26 +29,20 @@
 #include <stdio.h>
 #include <signal.h>
 
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/Xproto.h>
-#include <X11/Xatom.h>
-#include <X11/Intrinsic.h>
-#include <X11/keysym.h>
-
 #include "libs/fvwmlib.h"
-#include "libs/FScreen.h"
 #include "libs/FShape.h"
 #include "libs/Module.h"
 #include "libs/Colorset.h"
 #include "libs/Graphics.h"
 #include "fvwm/fvwm.h"
 #include "libs/PictureGraphics.h"
+#include "libs/FEvent.h"
 #include "FvwmPager.h"
 
 
 extern ScreenInfo Scr;
 extern Display *dpy;
+extern char *monitor_to_track;
 
 Pixel back_pix, fore_pix, hi_pix;
 Pixel focus_pix;
@@ -217,93 +210,58 @@ static void discard_events(long event_type, Window w, XEvent *last_ev)
 	return;
 }
 
-/*
- *
- *  Procedure:
- *	CalcGeom - calculates the size and position of a mini-window
- *	given the real window size.
- *	You can always tell bad code by the size of the comments.
- */
 static void CalcGeom(PagerWindow *t, int win_w, int win_h,
 		     int *x_ret, int *y_ret, int *w_ret, int *h_ret)
 {
-  int virt, virt2, edge, edge2, size, page, over;
+	int wfull, hfull, xwin, ywin;
+	struct fpmonitor *m = fpmonitor_this();
 
-  /* coordinate of left hand edge on virtual desktop */
-  virt = Scr.Vx + t->x;
+	/* Only track windows on the appropriate monitor if per-monitor mode. */
+	if (monitor_to_track != NULL && t->m != m) {
+		*x_ret = 0;
+		*y_ret = 0;
+		*w_ret = 0;
+		*h_ret = 0;
+		return;
+	}
 
-  /* position of left hand edge of mini-window on pager window */
-  edge = (virt * win_w) / Scr.VWidth;
+	if (monitor_to_track == NULL) {
+		wfull = m->virtual_scr.VWidth;
+		hfull = m->virtual_scr.VHeight;
+		xwin = m->virtual_scr.Vx + t->x;
+		ywin = m->virtual_scr.Vy + t->y;
+	} else {
+		int page;
 
-  /* absolute coordinate of right hand edge on virtual desktop */
-  virt += t->width - 1;
+		/* Shrink geometry to current monitor. */
+		wfull = m->w * m->virtual_scr.VxPages;
+		hfull = m->h * m->virtual_scr.VyPages;
+		xwin = m->virtual_scr.Vx + t->x;
+		ywin = m->virtual_scr.Vy + t->y;
 
-  /* to calculate the right edge, mirror the window and use the same
-   * calculations as for the left edge for consistency. */
-  virt2 = Scr.VWidth - 1 - virt;
-  edge2 = (virt2 * win_w) / Scr.VWidth;
+		/* Adjust coordinates to be inside current monitor
+		 * Moves windows not in current monitor to the monitor's edge.
+		 */
+		page = xwin / m->virtual_scr.MyDisplayWidth;
+		xwin = page * m->w + min(m->w,
+			max(0, xwin % m->virtual_scr.MyDisplayWidth - m->x));
+		page = ywin / m->virtual_scr.MyDisplayHeight;
+		ywin = page * m->h + min(m->h,
+			max(0, ywin % m->virtual_scr.MyDisplayHeight - m->y));
+	}
+	/* fprintf(stderr, "Input: {Mon: %s, Wfull: %d, Xwin: %d, Hfull: %d, "
+	 *                "Ywin: %d}\n",
+	 *		m->name, wfull, xwin, hfull, ywin );
+	 */
 
-  /* then mirror it back to get the real coordinate */
-  edge2 = win_w - 1 - edge2;
-
-  /* Calculate the mini-window's width by subtracting its LHS
-   * from its RHS. This theoretically means that the width will
-   * vary slightly as the window travels around the screen, but
-   * this way ensures that the mini-windows in the pager match
-   * the actual screen layout. */
-  size = edge2 - edge + 1;
-
-  /* Make size big enough to be visible */
-  if (size < MinSize) {
-    size = MinSize;
-    /* this mini-window has to be grown to be visible
-     * which way it grows depends on some magic:
-     * normally it will grow right but if the window is on the right hand
-     * edge of a page it should be grown left so that the pager looks better */
-
-    /* work out the page that the right hand edge is on */
-    page = virt / Scr.MyDisplayWidth;
-
-    /* if the left edge is on the same page then possibly move it left */
-    if (page == ((virt - t->width + 1) / Scr.MyDisplayWidth)) {
-      /* calculate how far the mini-window right edge overlaps the page line */
-      /* beware that the "over" is actually one greater than on screen, but
-	 this discrepancy is catered for in the next two lines */
-      over = edge + size - ((page + 1) * win_w * Scr.MyDisplayWidth) /
-	Scr.VWidth;
-
-      /* if the mini-window right edge is beyond the mini-window pager grid */
-      if (over > 0) {
-	/* move it left by the amount of pager grid overlap (!== the growth) */
-	edge -= over;
-      }
-    }
-  }
-  /* fill in return values */
-  *x_ret = edge;
-  *w_ret = size;
-
-  /* same code for y axis */
-  virt = Scr.Vy + t->y;
-  edge = (virt * win_h) / Scr.VHeight;
-  virt += t->height - 1;
-  virt2 = Scr.VHeight - 1 - virt;
-  edge2 = (virt2 * win_h) / Scr.VHeight;
-  edge2 = win_h - 1 - edge2;
-  size = edge2 - edge + 1;
-  if (size < MinSize)
-  {
-    size = MinSize;
-    page = virt / Scr.MyDisplayHeight;
-    if (page == ((virt - t->height + 1) / Scr.MyDisplayHeight)) {
-      over = edge + size - ((page + 1) * win_h * Scr.MyDisplayHeight) /
-	Scr.VHeight;
-      if (over > 0)
-	edge -= over;
-    }
-  }
-  *y_ret = edge;
-  *h_ret = size;
+	/* Due to rounding some windows may appear slightly off the page
+	 * edges when they are not. Should add some cleanup/snapping
+	 * code to help improve the visual effect.
+	 */
+	*x_ret = (xwin * win_w) / wfull;
+	*w_ret = max(MinSize, (t->width * win_w) / wfull);
+	*y_ret = (ywin * win_h) / hfull;
+	*h_ret = max(MinSize, (t->height * win_h) / hfull);
 }
 
 /*
@@ -502,6 +460,7 @@ void initialize_pager(void)
   XGCValues gcv;
   char dash_list[2];
   FlocaleFont *balloon_font;
+  struct fpmonitor *mon = fpmonitor_this();
 
   /* I don't think that this is necessary - just let pager die */
   /* domivogt (07-mar-1999): But it is! A window being moved in the pager
@@ -584,9 +543,8 @@ void initialize_pager(void)
 				  fore_pix,back_pix,Pdepth);
   }
 
-
-  n = Scr.VxMax / Scr.MyDisplayWidth;
-  m = Scr.VyMax / Scr.MyDisplayHeight;
+  n = mon->virtual_scr.VxMax / (mon->x + mon->w); //mon->virtual_scr.MyDisplayWidth;
+  m = mon->virtual_scr.VyMax / (mon->y + mon->h); //mon->virtual_scr.MyDisplayHeight;
 
   /* Size the window */
   if(Rows < 0)
@@ -625,6 +583,10 @@ void initialize_pager(void)
   sizehints.height_inc = Rows*(m+1);
   sizehints.base_width = Columns * n + Columns - 1;
   sizehints.base_height = Rows * (m + label_h + 1) - 1;
+
+  int vWidth  = mon->virtual_scr.VxPages * (mon->x + mon->w);
+  int vHeight = mon->virtual_scr.VyPages * (mon->y + mon->h);
+
   if (window_w > 0)
   {
     window_w = (window_w - sizehints.base_width) / sizehints.width_inc;
@@ -632,7 +594,7 @@ void initialize_pager(void)
   }
   else
   {
-    window_w = Columns * (Scr.VWidth / Scr.VScale + n) + Columns - 1;
+    window_w = Columns * (vWidth / Scr.VScale + n) + Columns - 1;
   }
   if (window_h > 0)
   {
@@ -641,7 +603,7 @@ void initialize_pager(void)
   }
   else
   {
-    window_h = Rows * (Scr.VHeight / Scr.VScale + m + label_h + 1) - 1;
+    window_h = Rows * (vHeight / Scr.VScale + m + label_h + 1) - 1;
   }
   desk_w = (window_w - Columns + 1) / Columns;
   desk_h = (window_h - Rows * label_h - Rows + 1) / Rows;
@@ -657,23 +619,23 @@ void initialize_pager(void)
       &screen_g.x, &screen_g.y, &screen_g.width, &screen_g.height);
     if (window_w + window_x > screen_g.x + screen_g.width)
     {
-      window_x = screen_g.x + screen_g.width - Scr.MyDisplayWidth;
+      window_x = screen_g.x + screen_g.width - mon->w; //mon->virtual_scr.MyDisplayWidth;
       xneg = 1;
     }
     if (window_h + window_y > screen_g.y + screen_g.height)
     {
-      window_y = screen_g.y + screen_g.height - Scr.MyDisplayHeight;
+      window_y = screen_g.y + screen_g.height - mon->h; //mon->virtual_scr.MyDisplayHeight;
       yneg = 1;
     }
   }
   if (xneg)
   {
     sizehints.win_gravity = NorthEastGravity;
-    window_x = Scr.MyDisplayWidth - window_w + window_x;
+    window_x = monitor_get_all_widths() - window_w + window_x;
   }
   if (yneg)
   {
-    window_y = Scr.MyDisplayHeight - window_h + window_y;
+    window_y = monitor_get_all_heights() - window_h + window_y;
     if(sizehints.win_gravity == NorthEastGravity)
       sizehints.win_gravity = SouthEastGravity;
     else
@@ -757,11 +719,11 @@ void initialize_pager(void)
   if (icon_x != -10000)
   {
     if (icon_xneg)
-      icon_x = Scr.MyDisplayWidth + icon_x - icon_w;
+      icon_x = mon->virtual_scr.MyDisplayWidth + icon_x - icon_w;
     if (icon_y != -10000)
     {
       if (icon_yneg)
-	icon_y = Scr.MyDisplayHeight + icon_y - icon_h;
+	icon_y = mon->virtual_scr.MyDisplayHeight + icon_y - icon_h;
     }
     else
     {
@@ -983,12 +945,13 @@ void UpdateWindowShape(void)
   {
     int i, j, cnt, shape_count, x_pos, y_pos;
     XRectangle *shape;
+    struct fpmonitor *m = fpmonitor_this();
 
     if (!ShapeLabels || !uselabel || label_h<=0)
       return;
 
     shape_count =
-      ndesks + ((Scr.CurrentDesk < desk1 || Scr.CurrentDesk >desk2) ? 0 : 1);
+      ndesks + ((m->virtual_scr.CurrentDesk < desk1 || m->virtual_scr.CurrentDesk >desk2) ? 0 : 1);
 
     shape = (XRectangle *)alloca (shape_count * sizeof (XRectangle));
 
@@ -1010,7 +973,7 @@ void UpdateWindowShape(void)
 	  shape[cnt].width = desk_w + 1;
 	  shape[cnt].height = desk_h + 2;
 
-	  if (cnt == Scr.CurrentDesk - desk1)
+	  if (cnt == m->virtual_scr.CurrentDesk - desk1)
 	  {
 	    shape[ndesks].x = x_pos;
 	    shape[ndesks].y =
@@ -1049,6 +1012,7 @@ void DispatchEvent(XEvent *Event)
   Bool do_move_page = False;
   short dx = 0;
   short dy = 0;
+  struct fpmonitor *mon = fpmonitor_this();
 
   switch(Event->xany.type)
   {
@@ -1176,8 +1140,8 @@ void DispatchEvent(XEvent *Event)
     else if (Event->xbutton.button == 3)
     {
       /* save initial virtual desk position for drag */
-      MyVx=Scr.Vx;
-      MyVy=Scr.Vy;
+      MyVx=mon->virtual_scr.Vx;
+      MyVy=mon->virtual_scr.Vy;
       for(i=0;i<ndesks;i++)
       {
 	if(Event->xany.window == Desks[i].w)
@@ -1187,8 +1151,8 @@ void DispatchEvent(XEvent *Event)
 	  {
 	    /* pointer is on a different screen - that's okay here */
 	  }
-	  Scroll(desk_w, desk_h, x, y, Scr.CurrentDesk, False);
-	  if (Scr.CurrentDesk != i + desk1)
+	  Scroll(desk_w, desk_h, x, y, mon->virtual_scr.CurrentDesk, False);
+	  if (mon->virtual_scr.CurrentDesk != i + desk1)
 	  {
 	    Wait = 0;
 	    SwitchToDesk(i);
@@ -1373,6 +1337,7 @@ void ReConfigure(void)
   int old_ww;
   int old_wh;
   int is_size_changed;
+  struct fpmonitor *mon = fpmonitor_this();
 
   old_ww = window_w;
   old_wh = window_h;
@@ -1383,10 +1348,10 @@ void ReConfigure(void)
   }
   is_size_changed = (old_ww != window_w || old_wh != window_h);
 
-  n1 = Scr.Vx / Scr.MyDisplayWidth;
-  m1 = Scr.Vy / Scr.MyDisplayHeight;
-  n = Scr.VxMax / Scr.MyDisplayWidth;
-  m = Scr.VyMax / Scr.MyDisplayHeight;
+  n1 = mon->virtual_scr.Vx / mon->virtual_scr.MyDisplayWidth;
+  m1 = mon->virtual_scr.Vy / mon->virtual_scr.MyDisplayHeight;
+  n = mon->virtual_scr.VxMax / mon->virtual_scr.MyDisplayWidth;
+  m = mon->virtual_scr.VyMax / mon->virtual_scr.MyDisplayHeight;
 
   sizehints.width_inc = Columns * (n + 1);
   sizehints.height_inc = Rows * (m + 1);
@@ -1411,8 +1376,8 @@ void ReConfigure(void)
 
   XSetWMNormalHints(dpy,Scr.Pager_w,&sizehints);
 
-  x = (desk_w - n) * Scr.Vx / Scr.VWidth + n1;
-  y = (desk_h - m) * Scr.Vy / Scr.VHeight + m1;
+  x = (desk_w - n) * mon->virtual_scr.Vx / mon->virtual_scr.VWidth + n1;
+  y = (desk_h - m) * mon->virtual_scr.Vy / mon->virtual_scr.VHeight + m1;
 
   for(k=0;k<Rows;k++)
   {
@@ -1435,7 +1400,7 @@ void ReConfigure(void)
 	}
 	if (HilightDesks)
 	{
-	  if(i == Scr.CurrentDesk - desk1)
+	  if(i == mon->virtual_scr.CurrentDesk - desk1)
 	    XMoveResizeWindow(dpy, Desks[i].CPagerWin, x,y,w,h);
 	  else
 	    XMoveResizeWindow(dpy, Desks[i].CPagerWin, -32768, -32768,w,h);
@@ -1468,9 +1433,10 @@ void update_pr_transparent_subwindows(int i)
 	int cset;
 	int n,m,w,h;
 	PagerWindow *t;
+	struct fpmonitor *mon = fpmonitor_this();
 
-	n = Scr.VxMax / Scr.MyDisplayWidth;
-	m = Scr.VyMax / Scr.MyDisplayHeight;
+	n = mon->virtual_scr.VxMax / mon->virtual_scr.MyDisplayWidth;
+	m = mon->virtual_scr.VyMax / mon->virtual_scr.MyDisplayHeight;
 	w = (desk_w - n)/(n+1);
 	h = (desk_h - m)/(m+1);
 
@@ -1513,9 +1479,10 @@ void update_pr_transparent_windows(void)
 	int i,j,k,cset;
 	int n,m,w,h;
 	PagerWindow *t;
+	struct fpmonitor *mon = fpmonitor_this();
 
-	n = Scr.VxMax / Scr.MyDisplayWidth;
-	m = Scr.VyMax / Scr.MyDisplayHeight;
+	n = mon->virtual_scr.VxMax / mon->virtual_scr.MyDisplayWidth;
+	m = mon->virtual_scr.VyMax / mon->virtual_scr.MyDisplayHeight;
 	w = (desk_w - n)/(n+1);
 	h = (desk_h - m)/(m+1);
 
@@ -1587,15 +1554,16 @@ void MovePage(Bool is_new_desk)
   XTextProperty name;
   char str[100],*sptr;
   static int icon_desk_shown = -1000;
+  struct fpmonitor *mon = fpmonitor_this();
 
   Wait = 0;
-  n1 = Scr.Vx/Scr.MyDisplayWidth;
-  m1 = Scr.Vy/Scr.MyDisplayHeight;
-  n = Scr.VxMax / Scr.MyDisplayWidth;
-  m = Scr.VyMax / Scr.MyDisplayHeight;
+  n1 = mon->virtual_scr.Vx/mon->virtual_scr.MyDisplayWidth;
+  m1 = mon->virtual_scr.Vy/mon->virtual_scr.MyDisplayHeight;
+  n = mon->virtual_scr.VxMax / mon->virtual_scr.MyDisplayWidth;
+  m = mon->virtual_scr.VyMax / mon->virtual_scr.MyDisplayHeight;
 
-  x = (desk_w - n) * Scr.Vx / Scr.VWidth + n1;
-  y = (desk_h - m) * Scr.Vy / Scr.VHeight + m1;
+  x = (desk_w - n) * mon->virtual_scr.Vx / mon->virtual_scr.VWidth + n1;
+  y = (desk_h - m) * mon->virtual_scr.Vy / mon->virtual_scr.VHeight + m1;
   w = (desk_w - n)/(n+1);
   h = (desk_h - m)/(m+1);
 
@@ -1603,7 +1571,7 @@ void MovePage(Bool is_new_desk)
   {
     if (HilightDesks)
     {
-      if(i == Scr.CurrentDesk - desk1)
+      if(i == mon->virtual_scr.CurrentDesk - desk1)
       {
 	XMoveWindow(dpy, Desks[i].CPagerWin, x,y);
 	XLowerWindow(dpy,Desks[i].CPagerWin);
@@ -1625,15 +1593,15 @@ void MovePage(Bool is_new_desk)
 
   ReConfigureIcons(!is_new_desk);
 
-  if(Scr.CurrentDesk != icon_desk_shown)
+  if(mon->virtual_scr.CurrentDesk != icon_desk_shown)
   {
-    icon_desk_shown = Scr.CurrentDesk;
+    icon_desk_shown = mon->virtual_scr.CurrentDesk;
 
-    if((Scr.CurrentDesk >= desk1)&&(Scr.CurrentDesk <=desk2))
-      sptr = Desks[Scr.CurrentDesk -desk1].label;
+    if((mon->virtual_scr.CurrentDesk >= desk1)&&(mon->virtual_scr.CurrentDesk <=desk2))
+      sptr = Desks[mon->virtual_scr.CurrentDesk -desk1].label;
     else
     {
-      sprintf(str, "GotoDesk %d", Scr.CurrentDesk);
+      sprintf(str, "GotoDesk %s %d", mon->name, mon->virtual_scr.CurrentDesk);
       sptr = &str[0];
     }
 
@@ -1664,17 +1632,18 @@ void ReConfigureIcons(Bool do_reconfigure_desk_only)
 {
   PagerWindow *t;
   int x, y, w, h;
+  struct fpmonitor *mon = fpmonitor_this();
 
   for (t = Start; t != NULL; t = t->next)
   {
-    if (do_reconfigure_desk_only && t->desk != Scr.CurrentDesk)
+    if (do_reconfigure_desk_only && t->desk != mon->virtual_scr.CurrentDesk)
       continue;
     CalcGeom(t, icon_w, icon_h, &x, &y, &w, &h);
     t->icon_view_x = x;
     t->icon_view_y = y;
     t->icon_view_width = w;
     t->icon_view_height = h;
-    if(Scr.CurrentDesk == t->desk)
+    if(mon->virtual_scr.CurrentDesk == t->desk)
       XMoveResizeWindow(dpy, t->IconView, x, y, w, h);
     else
       XMoveResizeWindow(dpy, t->IconView, -32768, -32768, w, h);
@@ -1693,6 +1662,10 @@ void DrawGrid(int desk, int erase, Window ew, XRectangle *r)
 	int cs;
 	XRectangle bound;
 	Region region = 0;
+	struct fpmonitor *mon = fpmonitor_this();
+
+	if (mon == NULL)
+		return;
 
 	if((desk < 0 ) || (desk >= ndesks))
 		return;
@@ -1700,33 +1673,33 @@ void DrawGrid(int desk, int erase, Window ew, XRectangle *r)
 	/* desk grid */
 	if (!ew || ew == Desks[desk].w)
 	{
-		x = Scr.MyDisplayWidth;
+		x = mon->virtual_scr.MyDisplayWidth;
 		y1 = 0;
 		y2 = desk_h;
-		while (x < Scr.VWidth)
+		while (x < mon->virtual_scr.VWidth)
 		{
-			x1 = (x * desk_w) / Scr.VWidth;
+			x1 = (x * desk_w) / mon->virtual_scr.VWidth;
 			if (!use_no_separators)
 			{
 				XDrawLine(
 					dpy,Desks[desk].w,Desks[desk].DashedGC,
 					x1,y1,x1,y2);
 			}
-			x += Scr.MyDisplayWidth;
+			x += mon->virtual_scr.MyDisplayWidth;
 		}
-		y = Scr.MyDisplayHeight;
+		y = mon->virtual_scr.MyDisplayHeight;
 		x1 = 0;
 		x2 = desk_w;
-		while(y < Scr.VHeight)
+		while(y < mon->virtual_scr.VHeight)
 		{
-			y1 = (y * desk_h) / Scr.VHeight;
+			y1 = (y * desk_h) / mon->virtual_scr.VHeight;
 			if (!use_no_separators)
 			{
 				XDrawLine(
 					dpy,Desks[desk].w,Desks[desk].DashedGC,
 					x1,y1,x2,y1);
 			}
-			y += Scr.MyDisplayHeight;
+			y += mon->virtual_scr.MyDisplayHeight;
 		}
 	}
 
@@ -1757,7 +1730,7 @@ void DrawGrid(int desk, int erase, Window ew, XRectangle *r)
 	{
 		erase = True;
 	}
-	if(((Scr.CurrentDesk - desk1) == desk) && !ShapeLabels)
+	if(((mon->virtual_scr.CurrentDesk - desk1) == desk) && !ShapeLabels)
 	{
 		if (uselabel)
 		{
@@ -1789,7 +1762,7 @@ void DrawGrid(int desk, int erase, Window ew, XRectangle *r)
 	{
 		FwinString->str = ptr;
 		FwinString->win = Desks[desk].title_w;
-		if(desk == (Scr.CurrentDesk - desk1))
+		if(desk == (mon->virtual_scr.CurrentDesk - desk1))
 		{
 			cs = Desks[desk].highcolorset;
 			FwinString->gc = Desks[desk].rvGC;
@@ -1840,10 +1813,11 @@ void DrawIconGrid(int erase)
 {
   int y, y1, y2, x, x1, x2,w,h,n,m,n1,m1;
   int i;
+  struct fpmonitor *mon = fpmonitor_this();
 
   if(erase)
   {
-    int tmp=(Scr.CurrentDesk - desk1);
+    int tmp=(mon->virtual_scr.CurrentDesk - desk1);
 
     if ((tmp < 0) || (tmp >= ndesks))
     {
@@ -1868,38 +1842,38 @@ void DrawIconGrid(int erase)
     XClearWindow(dpy,icon_win);
   }
 
-  x = Scr.MyDisplayWidth;
+  x = mon->virtual_scr.MyDisplayWidth;
   y1 = 0;
   y2 = icon_h;
-  while(x < Scr.VWidth)
+  while(x < mon->virtual_scr.VWidth)
   {
-    x1 = x * icon_w / Scr.VWidth;
+    x1 = x * icon_w / mon->virtual_scr.VWidth;
     if (!use_no_separators)
       for(i=0;i<ndesks;i++)
 	XDrawLine(dpy,icon_win,Desks[i].DashedGC,x1,y1,x1,y2);
-    x += Scr.MyDisplayWidth;
+    x += mon->virtual_scr.MyDisplayWidth;
   }
 
-  y = Scr.MyDisplayHeight;
+  y = mon->virtual_scr.MyDisplayHeight;
   x1 = 0;
   x2 = icon_w;
-  while(y < Scr.VHeight)
+  while(y < mon->virtual_scr.VHeight)
   {
-    y1 = y * icon_h / Scr.VHeight;
+    y1 = y * icon_h / mon->virtual_scr.VHeight;
     if (!use_no_separators)
       for(i=0;i<ndesks;i++)
 	XDrawLine(dpy,icon_win,Desks[i].DashedGC,x1,y1,x2,y1);
-    y += Scr.MyDisplayHeight;
+    y += mon->virtual_scr.MyDisplayHeight;
   }
-  n1 = Scr.Vx / Scr.MyDisplayWidth;
-  m1 = Scr.Vy / Scr.MyDisplayHeight;
-  n = Scr.VxMax / Scr.MyDisplayWidth;
-  m = Scr.VyMax / Scr.MyDisplayHeight;
+  n1 = mon->virtual_scr.Vx / mon->virtual_scr.MyDisplayWidth;
+  m1 = mon->virtual_scr.Vy / mon->virtual_scr.MyDisplayHeight;
+  n = mon->virtual_scr.VxMax / mon->virtual_scr.MyDisplayWidth;
+  m = mon->virtual_scr.VyMax / mon->virtual_scr.MyDisplayHeight;
   w = (icon_w - n) / (n + 1);
   h = (icon_h - m) / (m + 1);
 
-  x = (icon_w - n) * Scr.Vx / Scr.VWidth + n1;
-  y = (icon_h - m) * Scr.Vy / Scr.VHeight + m1;
+  x = (icon_w - n) * mon->virtual_scr.Vx / mon->virtual_scr.VWidth + n1;
+  y = (icon_h - m) * mon->virtual_scr.Vy / mon->virtual_scr.VHeight + m1;
 
   if (HilightDesks)
   {
@@ -1920,33 +1894,52 @@ void DrawIconGrid(int erase)
 
 void SwitchToDesk(int Desk)
 {
-  char command[256];
+	char command[256];
+	struct fpmonitor *m = fpmonitor_this();
 
-  sprintf(command, "GotoDesk 0 %d", Desk + desk1);
-  SendText(fd,command,0);
+	sprintf(command, "GotoDesk %s 0 %d", monitor_to_track ? m->name : "",
+			Desk + desk1);
+	fprintf(stderr, "%s: cmd: %s\n", __func__, command);
+	SendText(fd,command,0);
 }
 
 
 void SwitchToDeskAndPage(int Desk, XEvent *Event)
 {
   char command[256];
+  struct fpmonitor *mon = fpmonitor_this();
 
-  if (Scr.CurrentDesk != (Desk + desk1))
+  if (mon->virtual_scr.CurrentDesk != (Desk + desk1))
   {
     int vx, vy;
     /* patch to let mouse button 3 change desks and do not cling to a page */
-    vx = Event->xbutton.x * Scr.VWidth / (desk_w * Scr.MyDisplayWidth);
-    vy = Event->xbutton.y * Scr.VHeight / (desk_h * Scr.MyDisplayHeight);
-    Scr.Vx = vx * Scr.MyDisplayWidth;
-    Scr.Vy = vy * Scr.MyDisplayHeight;
-    sprintf(command, "GotoDeskAndPage %d %d %d", Desk + desk1, vx, vy);
+    if (desk_w == 0)
+      vx = 0;
+    else
+      vx = Event->xbutton.x * mon->virtual_scr.VWidth / (desk_w * mon->virtual_scr.MyDisplayWidth);
+    if (desk_h == 0)
+      vy = 0;
+    else
+      vy = Event->xbutton.y * mon->virtual_scr.VHeight / (desk_h * mon->virtual_scr.MyDisplayHeight);
+    mon->virtual_scr.Vx = vx * mon->virtual_scr.MyDisplayWidth;
+    mon->virtual_scr.Vy = vy * mon->virtual_scr.MyDisplayHeight;
+    sprintf(command, "GotoDeskAndPage %s %d %d %d",
+		monitor_to_track ? mon->name : "", Desk + desk1, vx, vy);
+    fprintf(stderr, "%s: cmd: %s\n", __func__, command);
     SendText(fd, command, 0);
 
   }
   else
   {
-    int x = Event->xbutton.x * Scr.VWidth / (desk_w * Scr.MyDisplayWidth);
-    int y = Event->xbutton.y * Scr.VHeight / (desk_h * Scr.MyDisplayHeight);
+    int x, y;
+    if (desk_w == 0)
+      x = 0;
+    else
+      x = Event->xbutton.x * mon->virtual_scr.VWidth / (desk_w * mon->virtual_scr.MyDisplayWidth);
+    if (desk_h == 0)
+      y = 0;
+    else
+      y = Event->xbutton.y * mon->virtual_scr.VHeight / (desk_h * mon->virtual_scr.MyDisplayHeight);
 
     /* Fix for buggy XFree86 servers that report button release events
      * incorrectly when moving fast. Not perfect, but should at least prevent
@@ -1955,11 +1948,12 @@ void SwitchToDeskAndPage(int Desk, XEvent *Event)
       x = 0;
     if (y < 0)
       y = 0;
-    if (x * Scr.MyDisplayWidth > Scr.VxMax)
-      x = Scr.VxMax / Scr.MyDisplayWidth;
-    if (y * Scr.MyDisplayHeight > Scr.VyMax)
-      y = Scr.VyMax / Scr.MyDisplayHeight;
-    sprintf(command, "GotoPage %d %d", x, y);
+    if (x * mon->virtual_scr.MyDisplayWidth > mon->virtual_scr.VxMax)
+      x = mon->virtual_scr.VxMax / mon->virtual_scr.MyDisplayWidth;
+    if (y * mon->virtual_scr.MyDisplayHeight > mon->virtual_scr.VyMax)
+      y = mon->virtual_scr.VyMax / mon->virtual_scr.MyDisplayHeight;
+    sprintf(command, "GotoPage %s %d %d", monitor_to_track ? mon->name : "", x, y);
+    fprintf(stderr, "%s: cmd: %s\n", __func__, command);
     SendText(fd, command, 0);
   }
   Wait = 1;
@@ -1968,10 +1962,13 @@ void SwitchToDeskAndPage(int Desk, XEvent *Event)
 void IconSwitchPage(XEvent *Event)
 {
   char command[34];
+  struct fpmonitor *mon = fpmonitor_this();
 
-  sprintf(command,"GotoPage %d %d",
-	  Event->xbutton.x * Scr.VWidth / (icon_w * Scr.MyDisplayWidth),
-	  Event->xbutton.y * Scr.VHeight / (icon_h * Scr.MyDisplayHeight));
+  sprintf(command,"GotoPage %s %d %d",
+	  monitor_to_track ? mon->name : "",
+	  Event->xbutton.x * mon->virtual_scr.VWidth / (icon_w * mon->virtual_scr.MyDisplayWidth),
+	  Event->xbutton.y * mon->virtual_scr.VHeight / (icon_h * mon->virtual_scr.MyDisplayHeight));
+  fprintf(stderr, "%s: cmd: %s\n", __func__, command);
   SendText(fd, command, 0);
   Wait = 1;
 }
@@ -1982,6 +1979,7 @@ void AddNewWindow(PagerWindow *t)
 	unsigned long valuemask;
 	XSetWindowAttributes attributes;
 	int i, x, y, w, h;
+	struct fpmonitor	*mon = fpmonitor_this();
 
 	i = t->desk - desk1;
 	CalcGeom(t, desk_w, desk_h, &x, &y, &w, &h);
@@ -2000,6 +1998,9 @@ void AddNewWindow(PagerWindow *t)
 
 	if ((i >= 0) && (i < ndesks))
 	{
+		if (monitor_to_track != NULL && (t->m != NULL && t->m != mon))
+			return;
+
 		t->PagerView = XCreateWindow(
 			dpy,Desks[i].w, x, y, w, h, 0, CopyFromParent,
 			InputOutput, CopyFromParent, valuemask, &attributes);
@@ -2031,7 +2032,7 @@ void AddNewWindow(PagerWindow *t)
 	t->icon_view_y = y;
 	t->icon_view_width = w;
 	t->icon_view_height = h;
-	if(Scr.CurrentDesk != t->desk)
+	if(mon->virtual_scr.CurrentDesk != t->desk)
 	{
 		x = -32768;
 		y = -32768;
@@ -2045,7 +2046,7 @@ void AddNewWindow(PagerWindow *t)
 			dpy, t->IconView, w, h, &Colorset[windowcolorset],
 			Pdepth, Scr.NormalGC, True);
 	}
-	if(Scr.CurrentDesk == t->desk)
+	if(mon->virtual_scr.CurrentDesk == t->desk)
 	{
 		XGrabButton(
 			dpy, 2, AnyModifier, t->IconView, True,
@@ -2077,6 +2078,7 @@ void ChangeDeskForWindow(PagerWindow *t,long newdesk)
 {
   int i, x, y, w, h;
   Bool size_changed = False;
+  struct fpmonitor *mon = fpmonitor_this();
 
   i = newdesk - desk1;
   if(t->PagerView == None)
@@ -2122,7 +2124,7 @@ void ChangeDeskForWindow(PagerWindow *t,long newdesk)
   t->icon_view_y = y;
   t->icon_view_width = w;
   t->icon_view_height = h;
-  if(Scr.CurrentDesk != t->desk)
+  if(mon->virtual_scr.CurrentDesk != t->desk)
     XMoveResizeWindow(dpy,t->IconView,-32768,-32768,w,h);
   else
   {
@@ -2144,6 +2146,7 @@ void MoveResizePagerView(PagerWindow *t, Bool do_force_redraw)
   int x, y, w, h;
   Bool size_changed;
   Bool position_changed;
+  struct fpmonitor *mon = fpmonitor_this();
 
   if (UseSkipList && DO_SKIP_WINDOW_LIST(t) && t->myflags.is_mapped)
   {
@@ -2199,7 +2202,7 @@ void MoveResizePagerView(PagerWindow *t, Bool do_force_redraw)
   t->icon_view_y = y;
   t->icon_view_width = w;
   t->icon_view_height = h;
-  if (Scr.CurrentDesk == t->desk)
+  if (mon->virtual_scr.CurrentDesk == t->desk)
   {
     int cset;
 
@@ -2222,15 +2225,16 @@ void MoveResizePagerView(PagerWindow *t, Bool do_force_redraw)
 void MoveStickyWindow(Bool is_new_page, Bool is_new_desk)
 {
 	PagerWindow *t;
+	struct fpmonitor *mon = fpmonitor_this();
 
 	for (t = Start; t != NULL; t = t->next)
 	{
 		if (
-			is_new_desk && t->desk != Scr.CurrentDesk &&
+			is_new_desk && t->desk != mon->virtual_scr.CurrentDesk &&
 			((IS_ICONIFIED(t) && IS_ICON_STICKY_ACROSS_DESKS(t)) ||
 			 IS_STICKY_ACROSS_DESKS(t)))
 		{
-			ChangeDeskForWindow(t, Scr.CurrentDesk);
+			ChangeDeskForWindow(t, mon->virtual_scr.CurrentDesk);
 		}
 		else if (
 			is_new_page &&
@@ -2245,10 +2249,12 @@ void MoveStickyWindow(Bool is_new_page, Bool is_new_desk)
 
 void Hilight(PagerWindow *t, int on)
 {
-
   if(!t)
     return;
 
+  if (monitor_to_track != NULL && strcmp(t->m->name, monitor_to_track) != 0)
+	  return;
+
   if(Pdepth < 2)
   {
     if(on)
@@ -2321,16 +2327,17 @@ void Scroll(int window_w, int window_h, int x, int y, int Desk,
 	int sx;
 	int sy;
 	int adjx,adjy;
+	struct fpmonitor *mon = fpmonitor_this();
 
 	/* Desk < 0 means we want to scroll an icon window */
-	if (Desk >= 0 && Desk + desk1 != Scr.CurrentDesk)
+	if (Desk >= 0 && Desk + desk1 != mon->virtual_scr.CurrentDesk)
 	{
 		return;
 	}
 
 	/* center around mouse */
-	adjx = (desk_w / (1 + Scr.VxMax / Scr.MyDisplayWidth));
-	adjy = (desk_h / (1 + Scr.VyMax / Scr.MyDisplayHeight));
+	adjx = (desk_w / (1 + mon->virtual_scr.VxMax / mon->virtual_scr.MyDisplayWidth));
+	adjy = (desk_h / (1 + mon->virtual_scr.VyMax / mon->virtual_scr.MyDisplayHeight));
 	x -= adjx/2;
 	y -= adjy/2;
 
@@ -2357,11 +2364,11 @@ void Scroll(int window_w, int window_h, int x, int y, int Desk,
 	sy = 0;
 	if (window_w != 0)
 	{
-		sx = (x * Scr.VWidth / window_w - MyVx);
+		sx = (x * mon->virtual_scr.VWidth / window_w - MyVx);
 	}
 	if (window_h != 0)
 	{
-		sy = (y * Scr.VHeight / window_h - MyVy);
+		sy = (y * mon->virtual_scr.VHeight / window_h - MyVy);
 	}
 	MYFPRINTF((stderr,"[scroll]: %d %d %d %d %d %d\n", window_w, window_h,
 		   x, y, sx, sy));
@@ -2413,6 +2420,7 @@ void MoveWindow(XEvent *Event)
 	int JunkX, JunkY;
 	unsigned JunkMask;
 	int do_switch_desk_later = 0;
+	struct fpmonitor *mon = fpmonitor_this();
 
 	t = Start;
 	while (t != NULL && t->PagerView != Event->xbutton.subwindow)
@@ -2445,12 +2453,12 @@ void MoveWindow(XEvent *Event)
 		return;
 	}
 
-	n = Scr.VxMax / Scr.MyDisplayWidth;
-	m = Scr.VyMax / Scr.MyDisplayHeight;
-	n1 = (Scr.Vx + t->x) / Scr.MyDisplayWidth;
-	m1 = (Scr.Vy + t->y) / Scr.MyDisplayHeight;
-	wx = (Scr.Vx + t->x) * (desk_w - n) / Scr.VWidth + n1;
-	wy = (Scr.Vy + t->y) * (desk_h - m) / Scr.VHeight + m1;
+	n = mon->virtual_scr.VxMax / mon->virtual_scr.MyDisplayWidth;
+	m = mon->virtual_scr.VyMax / mon->virtual_scr.MyDisplayHeight;
+	n1 = (mon->virtual_scr.Vx + t->x) / mon->virtual_scr.MyDisplayWidth;
+	m1 = (mon->virtual_scr.Vy + t->y) / mon->virtual_scr.MyDisplayHeight;
+	wx = (mon->virtual_scr.Vx + t->x) * (desk_w - n) / mon->virtual_scr.VWidth + n1;
+	wy = (mon->virtual_scr.Vy + t->y) * (desk_h - m) / mon->virtual_scr.VHeight + m1;
 	wx1 = wx + (desk_w + 1) * (NewDesk % Columns);
 	wy1 = wy + label_h + (desk_h + label_h + 1) * (NewDesk / Columns);
 	if (LabelsBelow)
@@ -2516,7 +2524,7 @@ void MoveWindow(XEvent *Event)
 	}
 	if (KeepMoving)
 	{
-		NewDesk = Scr.CurrentDesk;
+		NewDesk = mon->virtual_scr.CurrentDesk;
 
 		if (NewDesk >= desk1 && NewDesk <= desk2)
 		{
@@ -2593,38 +2601,38 @@ void MoveWindow(XEvent *Event)
 		XTranslateCoordinates(dpy, Scr.Pager_w, Desks[NewDesk].w,
 				      x - x1, y - y1, &x2, &y2, &dumwin);
 
-		n1 = x2 * Scr.VWidth / (desk_w * Scr.MyDisplayWidth);
-		m1 = y2 * Scr.VHeight / (desk_h * Scr.MyDisplayHeight);
-		x = (x2 - n1) * Scr.VWidth / (desk_w - n) - Scr.Vx;
-		y = (y2 - m1) * Scr.VHeight / (desk_h - m) - Scr.Vy;
+		n1 = x2 * mon->virtual_scr.VWidth / (desk_w * mon->virtual_scr.MyDisplayWidth);
+		m1 = y2 * mon->virtual_scr.VHeight / (desk_h * mon->virtual_scr.MyDisplayHeight);
+		x = (x2 - n1) * mon->virtual_scr.VWidth / (desk_w - n) - mon->virtual_scr.Vx;
+		y = (y2 - m1) * mon->virtual_scr.VHeight / (desk_h - m) - mon->virtual_scr.Vy;
 		/* force onto desk */
-		if (x + t->frame_width + Scr.Vx < 0 )
+		if (x + t->frame_width + mon->virtual_scr.Vx < 0 )
 		{
-			x = -Scr.Vx - t->frame_width;
+			x = -mon->virtual_scr.Vx - t->frame_width;
 		}
-		if (y + t->frame_height + Scr.Vy < 0)
+		if (y + t->frame_height + mon->virtual_scr.Vy < 0)
 		{
-			y = -Scr.Vy - t->frame_height;
+			y = -mon->virtual_scr.Vy - t->frame_height;
 		}
-		if (x + Scr.Vx >= Scr.VWidth)
+		if (x + mon->virtual_scr.Vx >= mon->virtual_scr.VWidth)
 		{
-			x = Scr.VWidth - Scr.Vx - 1;
+			x = mon->virtual_scr.VWidth - mon->virtual_scr.Vx - 1;
 		}
-		if (y + Scr.Vy >= Scr.VHeight)
+		if (y + mon->virtual_scr.Vy >= mon->virtual_scr.VHeight)
 		{
-			y = Scr.VHeight - Scr.Vy - 1;
+			y = mon->virtual_scr.VHeight - mon->virtual_scr.Vy - 1;
 		}
 		if ((IS_ICONIFIED(t) && IS_ICON_STICKY_ACROSS_DESKS(t)) ||
 		   (IS_STICKY_ACROSS_DESKS(t)))
 		{
-			NewDesk = Scr.CurrentDesk - desk1;
-			if (x > Scr.MyDisplayWidth - 16)
+			NewDesk = mon->virtual_scr.CurrentDesk - desk1;
+			if (x > mon->virtual_scr.MyDisplayWidth - 16)
 			{
-				x = Scr.MyDisplayWidth - 16;
+				x = mon->virtual_scr.MyDisplayWidth - 16;
 			}
-			if (y > Scr.MyDisplayHeight - 16)
+			if (y > mon->virtual_scr.MyDisplayHeight - 16)
 			{
-				y = Scr.MyDisplayHeight - 16;
+				y = mon->virtual_scr.MyDisplayHeight - 16;
 			}
 			if (x + t->width < 16)
 			{
@@ -2640,13 +2648,13 @@ void MoveWindow(XEvent *Event)
 			if ((IS_ICONIFIED(t) && IS_ICON_STICKY_ACROSS_DESKS(t))
 			   || (IS_STICKY_ACROSS_DESKS(t)))
 			{
-				NewDesk = Scr.CurrentDesk - desk1;
-				if (t->desk != Scr.CurrentDesk)
+				NewDesk = mon->virtual_scr.CurrentDesk - desk1;
+				if (t->desk != mon->virtual_scr.CurrentDesk)
 				{
-					ChangeDeskForWindow(t,Scr.CurrentDesk);
+					ChangeDeskForWindow(t,mon->virtual_scr.CurrentDesk);
 				}
 			}
-			else if (NewDesk + desk1 != Scr.CurrentDesk)
+			else if (NewDesk + desk1 != mon->virtual_scr.CurrentDesk)
 			{
 				sprintf(command, "Silent MoveToDesk 0 %d",
 					NewDesk + desk1);
@@ -2686,7 +2694,7 @@ void MoveWindow(XEvent *Event)
 			SendText(fd, command, t->w);
 			t->desk = NewDesk + desk1;
 		}
-		if (Scr.CurrentDesk == t->desk)
+		if (mon->virtual_scr.CurrentDesk == t->desk)
 		{
 			XSync(dpy,0);
 			usleep(5000);
@@ -2804,7 +2812,7 @@ static void label_window_wrap(PagerWindow *t)
       }
     }
 
-    FwinString->str = safemalloc(next - cur + 1);
+    FwinString->str = fxmalloc(next - cur + 1);
     strncpy(FwinString->str, cur, next - cur);
     FwinString->str[next - cur] = 0;
 
@@ -2978,16 +2986,17 @@ void IconMoveWindow(XEvent *Event, PagerWindow *t)
 	Window JunkRoot, JunkChild;
 	int JunkX, JunkY;
 	unsigned JunkMask;
+	struct fpmonitor *mon = fpmonitor_this();
 
 	if (t == NULL || !t->allowed_actions.is_movable)
 	{
 		return;
 	}
 
-	n = Scr.VxMax / Scr.MyDisplayWidth;
-	m = Scr.VyMax / Scr.MyDisplayHeight;
-	n1 = (Scr.Vx + t->x) / Scr.MyDisplayWidth;
-	m1 = (Scr.Vy + t->y) / Scr.MyDisplayHeight;
+	n = mon->virtual_scr.VxMax / mon->virtual_scr.MyDisplayWidth;
+	m = mon->virtual_scr.VyMax / mon->virtual_scr.MyDisplayHeight;
+	n1 = (mon->virtual_scr.Vx + t->x) / mon->virtual_scr.MyDisplayWidth;
+	m1 = (mon->virtual_scr.Vy + t->y) / mon->virtual_scr.MyDisplayHeight;
 
 	XRaiseWindow(dpy, t->IconView);
 
@@ -3058,37 +3067,37 @@ void IconMoveWindow(XEvent *Event, PagerWindow *t)
 	{
 		x = x - x1;
 		y = y - y1;
-		n1 = x * Scr.VWidth / (icon_w * Scr.MyDisplayWidth);
-		m1 = y * Scr.VHeight / (icon_h * Scr.MyDisplayHeight);
-		x = (x - n1) * Scr.VWidth / (icon_w - n) - Scr.Vx;
-		y = (y - m1) * Scr.VHeight / (icon_h - m) - Scr.Vy;
+		n1 = x * mon->virtual_scr.VWidth / (icon_w * mon->virtual_scr.MyDisplayWidth);
+		m1 = y * mon->virtual_scr.VHeight / (icon_h * mon->virtual_scr.MyDisplayHeight);
+		x = (x - n1) * mon->virtual_scr.VWidth / (icon_w - n) - mon->virtual_scr.Vx;
+		y = (y - m1) * mon->virtual_scr.VHeight / (icon_h - m) - mon->virtual_scr.Vy;
 		/* force onto desk */
-		if (x + t->icon_width + Scr.Vx < 0 )
+		if (x + t->icon_width + mon->virtual_scr.Vx < 0 )
 		{
-			x = - Scr.Vx - t->icon_width;
+			x = - mon->virtual_scr.Vx - t->icon_width;
 		}
-		if (y + t->icon_height + Scr.Vy < 0)
+		if (y + t->icon_height + mon->virtual_scr.Vy < 0)
 		{
-			y = - Scr.Vy - t->icon_height;
+			y = - mon->virtual_scr.Vy - t->icon_height;
 		}
-		if (x + Scr.Vx >= Scr.VWidth)
+		if (x + mon->virtual_scr.Vx >= mon->virtual_scr.VWidth)
 		{
-			x = Scr.VWidth - Scr.Vx - 1;
+			x = mon->virtual_scr.VWidth - mon->virtual_scr.Vx - 1;
 		}
-		if (y + Scr.Vy >= Scr.VHeight)
+		if (y + mon->virtual_scr.Vy >= mon->virtual_scr.VHeight)
 		{
-			y = Scr.VHeight - Scr.Vy - 1;
+			y = mon->virtual_scr.VHeight - mon->virtual_scr.Vy - 1;
 		}
 		if ((IS_ICONIFIED(t) && IS_ICON_STICKY_ACROSS_DESKS(t)) ||
 		   IS_STICKY_ACROSS_DESKS(t))
 		{
-			if (x > Scr.MyDisplayWidth - 16)
+			if (x > mon->virtual_scr.MyDisplayWidth - 16)
 			{
-				x = Scr.MyDisplayWidth - 16;
+				x = mon->virtual_scr.MyDisplayWidth - 16;
 			}
-			if (y > Scr.MyDisplayHeight - 16)
+			if (y > mon->virtual_scr.MyDisplayHeight - 16)
 			{
-				y = Scr.MyDisplayHeight - 16;
+				y = mon->virtual_scr.MyDisplayHeight - 16;
 			}
 			if (x + t->width < 16)
 			{
@@ -3200,6 +3209,7 @@ void MapBalloonWindow(PagerWindow *t, Bool is_icon_view)
 	int x, y;
 	rectangle new_g;
 	int i;
+	struct fpmonitor *mon = fpmonitor_this();
 
 	if (!is_icon_view)
 	{
@@ -3279,7 +3289,7 @@ void MapBalloonWindow(PagerWindow *t, Bool is_icon_view)
 	}
 	/* too close to bottom ... make yoffset -ve */
 	else if ( new_g.y + new_g.height >
-		  Scr.MyDisplayHeight - (2 * BalloonBorderWidth) - 2 )
+		  mon->virtual_scr.MyDisplayHeight - (2 * BalloonBorderWidth) - 2 )
 	{
 		y = - BalloonYOffset + 1 - new_g.height -
 			(2 * BalloonBorderWidth);
@@ -3293,9 +3303,9 @@ void MapBalloonWindow(PagerWindow *t, Bool is_icon_view)
 	}
 	/* too close to right */
 	else if (new_g.x + new_g.width >
-		 Scr.MyDisplayWidth - (2 * BalloonBorderWidth) - 2 )
+		 mon->virtual_scr.MyDisplayWidth - (2 * BalloonBorderWidth) - 2 )
 	{
-		new_g.x = Scr.MyDisplayWidth - new_g.width -
+		new_g.x = mon->virtual_scr.MyDisplayWidth - new_g.width -
 			(2 * BalloonBorderWidth) - 2;
 	}
 	/* make changes to window */
@@ -3331,7 +3341,7 @@ static void InsertExpand(char **dest, char *s)
   if (!s || !*s)
     return;
   len = strlen(*dest) + strlen(s) + 1;
-  *dest = (char *)safemalloc(len);
+  *dest = fxmalloc(len);
   strcpy(*dest, tmp);
   free(tmp);
   strcat(*dest, s);
@@ -3343,7 +3353,7 @@ static void InsertExpand(char **dest, char *s)
    -- disching@fmi.uni-passau.de */
 char *GetBalloonLabel(const PagerWindow *pw,const char *fmt)
 {
-  char *dest = safestrdup("");
+  char *dest = fxstrdup("");
   const char *pos = fmt;
   char buffer[2];
 
@@ -3433,6 +3443,7 @@ void change_colorset(int colorset)
 {
   int i;
   PagerWindow *t;
+  struct fpmonitor *mon = fpmonitor_this();
 
   /* just in case */
   if (colorset < 0)
@@ -3447,7 +3458,7 @@ void change_colorset(int colorset)
       XSetForeground(dpy, Desks[i].rvGC, Colorset[colorset].fg);
       if (HilightDesks)
       {
-	if (uselabel && (i == Scr.CurrentDesk))
+	if (uselabel && (i == mon->virtual_scr.CurrentDesk))
 	{
 	  SetWindowBackground(
 	    dpy, Desks[i].title_w, desk_w, desk_h, &Colorset[colorset], Pdepth,
