@@ -42,7 +42,6 @@
 #include <X11/Xlib.h>
 
 #include "libs/fvwmlib.h"
-#include "libs/FScreen.h"
 #include "libs/Module.h"
 #include "fvwm/fvwm.h"
 #include "libs/vpacket.h"
@@ -183,11 +182,11 @@ int get_window(void)
     DeadPipe(0);
   else {
     cfgpacket = (struct ConfigWinPacket*) packet->body;
-    switch (packet->type) {
+    switch (packet->type &= ~M_EXTENDED_MSG) {
     case M_CONFIGURE_WINDOW:
       if (is_suitable_window(packet->body)) {
 	window_item *wi =
-	  (window_item*)safemalloc(sizeof( window_item ));
+	  fxmalloc(sizeof(window_item));
 	wi->frame = cfgpacket->frame;
 	wi->th = cfgpacket->title_height;
 	wi->bw = cfgpacket->border_width;
@@ -207,6 +206,7 @@ int get_window(void)
       fprintf(console,
 	"%s: internal inconsistency: unknown message 0x%08x\n",
 	module->name, (int)packet->type);
+      last = 1;
       break;
     }
   }
@@ -249,7 +249,7 @@ int atopixel(char *s, unsigned long f)
 void move_resize_raise_window(
 	window_item *wi, int x, int y, int w, int h)
 {
-	static char msg[64];
+	static char msg[78];
 	const char *ewmhiwa = do_ewmhiwa ?
 		"ewmhiwa" : "";
 
@@ -529,7 +529,8 @@ int main(int argc, char *argv[])
   module = ParseModuleArgs(argc,argv,0);
   if (module == NULL)
   {
-    fprintf(stderr,"FvwmRearrange: module should be executed by fvwm only\n");
+    fvwm_debug(__func__,
+               "FvwmRearrange: module should be executed by fvwm only\n");
     exit(-1);
   }
 
@@ -553,12 +554,6 @@ int main(int argc, char *argv[])
   GetConfigLine(fd, &config_line);
   while (config_line != NULL)
   {
-    if (strncasecmp(config_line, XINERAMA_CONFIG_STRING,
-		    sizeof(XINERAMA_CONFIG_STRING) - 1) == 0)
-    {
-      FScreenConfigureModule(
-	config_line + sizeof(XINERAMA_CONFIG_STRING) - 1);
-    }
     GetConfigLine(fd, &config_line);
   }
   FScreenGetScrRect(NULL, FSCREEN_CURRENT, &dx, &dy, &dwidth, &dheight);
